/**
*   Please feel free to use any fragment of the code in this file that you need
*   in your own work. As far as I am concerned, it's in the public domain. No
*   permission is necessary or required. Credit is always appreciated if you
*   use a large chunk or base a significant product on one of my examples,
*   but that's not required either.
*
*   This code is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
*      --- Joseph A. Huwaldt
**/

import java.awt.Component;
import javax.swing.JOptionPane;
import jahuwaldt.tools.ParamTools.*;


/**
*  Calculates the Mach number based on cone half-angle and
*  the oblique conical shock angle.
*
*  <p>  Modified by:  Joseph A. Huwaldt   </p>
*
*  @author   Joseph A. Huwaldt  Date: November 1, 2004
*  @version  November 7, 2004
**/
public class MachMethod extends ParamMethod implements SharedData, ParamVerifier {
	
	/**
	*  Constructor for this method.
	*
	*  @param   thetaC  Label for the "cone half-angle" parameter.
	*  @param   thetaS  Label for the "shock angle" parameter.
	**/
	public MachMethod(String thetaC, String thetaS) {
		inputLabels = new String[2];
		inputLabels[0] = thetaC;
		inputLabels[1] = thetaS;
	}
	
	
	/**
	*  Calculates the Mach number using the conical shock model object
	*  found in the SharedData interface.
	*
	*  @param   inputs  Array of input parameters.
	*                   [0=cone half-angle (thetaC), 1=oblique shock angle (thetaS)].
	*  @return  Mach number (M).
	**/
	public float calc(float[] inputs) throws Exception {
		float thetaC = inputs[0];
		float thetaS = inputs[1];
		
		float M = (float)conicalShock.freeStreamMach(thetaC, thetaS);
		
		return M;
	}

	/**
	*  Check to see if the input value is valid for this parameter.
	*
	*  @param   root   The root component containing the parameter input text field
	*                  for the parameter being verified.
	*  @param   value  The value of the parameter as input by the user.
	*  @return  If "value" is valid, it should be output.  If "value" is invalid or
	*           out of range, a valid value is output or Float.NaN if nothing else.
	*           If ParamInput (which calls this routine) recieves a value of Float.NaN,
	*           it will clear the parameter.
	**/
	public float check(Component root, float value) {
		float outValue = value;
		
		if (outValue <= 1) {
			outValue = Float.NaN;
			
			JOptionPane.showMessageDialog ( root,
                                "Free-stream Mach number must be supersonic.",
								"Out of Range",
								JOptionPane.ERROR_MESSAGE );
		}
		
		return outValue;
	}

}
