/**
*   Please feel free to use any fragment of the code in this file that you need
*   in your own work. As far as I am concerned, it's in the public domain. No
*   permission is necessary or required. Credit is always appreciated if you
*   use a large chunk or base a significant product on one of my examples,
*   but that's not required either.
*
*   This code is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
*      --- Joseph A. Huwaldt
**/

import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Date;
import java.text.NumberFormat;
import java.text.DecimalFormat;
import java.text.DateFormat;
import java.net.URL;
import java.net.MalformedURLException;
import java.io.File;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

import com.Ostermiller.util.Browser;

import net.roydesign.mac.MRJAdapter;
import net.roydesign.app.QuitJMenuItem;
import net.roydesign.app.AboutJMenuItem;
import net.roydesign.ui.StandardMacAboutFrame;

import jahuwaldt.swing.MDIApplication;
import jahuwaldt.swing.AppUtilities;
import jahuwaldt.swing.SplashScreen;

import jahuwaldt.util.BrowserHelp;

import jahuwaldt.tools.units.*;
import jahuwaldt.tools.ParamTools.*;


/**
*  Application that allows the user to compute the properties related
*  to a cone moving through air at supersonic speeds.
*
*  <p>  Modified by:  Joseph A. Huwaldt  </p>
*
*  @author  Joseph A. Huwaldt   Date:  October 24, 2004
*  @version October 29, 2008
**/
public class SupersonicConeApp extends MDIApplication implements SharedData {
	private static final short	fWidth = 680;					//	Application frame width & height.
	private static final short	fHeight = 480;
	private static final String aName = "Supersonic Cone";		//	Application/Applet Title or Name.
	private static final String	aVersion = "Version 1.0.2";		//	Version string.
	private static final String aAuthor = "Joseph A. Huwaldt";	//	Author string.
	private static final String aModDate = "October 29, 2008";	//	Modification date string.
	private static final String cwYear = "2004-2008";			//	Copywrite year.
	
	//  The ratio of specific heats of the gas.
	private static final float kGamma = 1.4F;
	
	//  The default file name given to this application.
	private static final String kAppName = "SupersonicConeApp";
	
	// Definition for our application's menus.
	private static final 
		String[][] fileMStrings = { { "New", "N", "handleNew" },
									{ "Open...", "O", null },
									{ null, null, null },
									{ "Close", "W", "handleClose" },
									{ "Save", "S", null },
									{ "Save As...", null, null } };

	private static final 
		String[][] editMStrings = { { "Undo", "Z", null },
									{ null, null, null },
									{ "Cut", "X", null },
									{ "Copy", "C", null },
									{ "Paste", "V", null } };

	//	An array of labels for the parameters in this program.
	private String[] pLabels;


    /**
    *  Entry point for this application.  This is where it starts up from.
    **/
	public static void main( String[] args ) {
	    try {
			// Set the system look and feel to hide that hideous Java LAF.
			AppUtilities.setSystemLAF();

			//  Initialize the Browser Opener class.
			Browser.init();
			
			// Create an instance of our application.
			SupersonicConeApp application = new SupersonicConeApp();
		
		} catch (ParamException e) {
			JOptionPane.showMessageDialog( null, e.getMessage(), "Parameter Error",
								JOptionPane.ERROR_MESSAGE );
			e.printStackTrace();
			System.exit(0);
		    
	    } catch (MalformedURLException e) {
			JOptionPane.showMessageDialog( null, e.getMessage(), "Parameter Display Error",
						JOptionPane.ERROR_MESSAGE );
			e.printStackTrace();
			System.exit(0);
			
	    } catch (Throwable e) {
			e.printStackTrace();
			AppUtilities.showException(null, "Unexpected Error",
					"Copy the following message and e-mail it to the author:", e);
			System.exit(0);
	    }
	    
	}
	
	/**
	*  Constructor for our standard atmosphere application.
	**/
	public SupersonicConeApp() throws Exception {
	    super(aName);
		
		//  Create the menu bar.
		JMenuBar menuBar = createMenuBar();
	    
		// Add the menu bar to this application.
		//  Cause the application to stay open when the last window is closed under MacOS.
		//  When the last window is closed, a menu bar will remain for the application with no
		//  window open.  This is standard MacOS behavior and will do nothing on other platforms.
		setFramelessJMenuBar( menuBar );
		
		//  Create a main window frame for this application.
		new AppWindow();
		
	}
	

	//-----------------------------------------------------------------------------------
	/**
	*  Handle the user choosing the "New..." from the File
	*  menu.  Creates and returns a reference to a new document window.
	**/
	public Frame handleNew( ActionEvent event ) {
		Frame window = null;
		try {
		
		    // Create an new instance of this application's window.
		    window = new AppWindow();
		    
		} catch (ParamException e) {
			JOptionPane.showMessageDialog( null, e.getMessage(), "Parameter Error",
								JOptionPane.ERROR_MESSAGE );
			e.printStackTrace();
		    
	    } catch (MalformedURLException e) {
			JOptionPane.showMessageDialog( null, e.getMessage(), "Parameter Display Error",
								JOptionPane.ERROR_MESSAGE );
			e.printStackTrace();
			
	    } catch (Exception e) {
			e.printStackTrace();
			AppUtilities.showException(null, "Unexpected Error",
								"Copy the following message and e-mail it to the author:", e);
	    }

		return window;
	}


	/**
	*  Initializes and displays the menus associated with this application.
	**/
	private JMenuBar createMenuBar() throws NoSuchMethodException {

		//  Create a menu bar.
		JMenuBar menuBar = new JMenuBar();
	
		// Set up the file menu.
		JMenu menu = AppUtilities.buildMenu( this, "File", fileMStrings );
		menuBar.add( menu );
		
		//  Add a Quit menu item.
		QuitJMenuItem quit = this.getQuitJMenuItem();
		quit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				handleQuit(e);
			}
		});
		if (!quit.isAutomaticallyPresent()) {
			menu.addSeparator();
			menu.add(quit);
		}
			

		// Set up the edit menu.
		menu = AppUtilities.buildMenu( this, "Edit", editMStrings );
		menuBar.add( menu );
		
		//	Create an about menu item.
		AboutJMenuItem about = this.getAboutJMenuItem();
		about.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				//  Use a generic icon since this app doesn't have a custom icon.
				Icon appIcon = UIManager.getIcon("OptionPane.informationIcon");
				
				//  Create a standard looking dialog box that displays the credits.
				String credits = "<html><body><b>Author:</b><br>" + aAuthor + "<br>";
				credits += "<a href=\"mailto:jhuwaldt@mac.com\">jhuwaldt@mac.com</a><br>";
				credits += "<br><b>Special thanks to:</b>";
				credits += "<br>MRJ Adapter: <a href=\"http://homepage.mac.com/sroy/mrjadapter/\">http://homepage.mac.com/sroy/mrjadapter/</a><br>";
				credits += "by Steve Roy:  <a href=\"mailto:sroy@mac.com\">sroy@mac.com</a><br>";
				credits += "<br>Browser Opener: <a href=\"http://ostermiller.org/utils/Browser.html\">http://ostermiller.org/utils/Browser.html</a><br>";
				credits += "by <a href=\"http://ostermiller.org/contact.pl?regarding=Java+Utilities\">Stephen Ostermiller</a>";
				credits += "<P ALIGN=CENTER><BR>" + aName + " comes with ABSOLUTELY NO WARRANTY;";
				credits += "<BR>This is free software, and you are welcome to redistribute ";
				credits += "it and it's source code under certain conditions.";
				credits += "<BR>Source code is available at:";
				credits += "<BR><a href=\"http://homepage.mac.com/jhuwaldt/java/\">";
				credits += "http://homepage.mac.com/jhuwaldt/java/</a></P>";
				credits += "</body></html>";
				StandardMacAboutFrame f = new StandardMacAboutFrame(aName, aVersion);
				f.setApplicationIcon(appIcon);
				f.setCopyright("Copyright " + cwYear + ", " + aAuthor);
				f.setCredits(credits, "text/html");
				f.setHyperlinkListener(new HyperlinkListener() {
					public void hyperlinkUpdate(HyperlinkEvent e) {
						if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
							try {
								//  Display the selected URL in the default browser for this platform.
								Browser.displayURL(e.getURL().toString());
							} catch (Exception ex) {
								AppUtilities.showException(null, "Unexpected Error",
									"Copy the following message and e-mail it to the author:", ex);
								ex.printStackTrace();
							}
						}
					}
				});
				f.setVisible(true);
			}
		});
		

		//	Create a "Help" menu for non-MacOS platforms.
		if (!about.isAutomaticallyPresent()) {
			//	Create Help menu.
			menu = new JMenu("Help");
			menuBar.add(menu);
			
			//	Add the "About" item to the Help menu.
			menu.add(about);
		}
		

		return menuBar;
	}


	private static int windowCount = 0;
	/**
	*  The main window frame for this application.
	**/
	private class AppWindow extends JFrame {
	
		//  A database to hold our parameters.
		private ParamDatabase pdb = new ParamDatabase();
	
		/**
		*  Construct a new application window instance.
		**/
		AppWindow() throws Exception {
			super (aName + (++windowCount>1?" " + String.valueOf(windowCount):""));
		
			//  Have the window dispose of itself when it closes.
			this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		
			//  Create a menu bar for this window.
			JMenuBar menuBar = createMenuBar();
			setJMenuBar(menuBar);
	    
			//  Set the ratio of specific heats for the gas we are using.
			SharedData.conicalShock.setGamma(kGamma);
			
			//	Create parameter objects for each of this program's parameters.
			initParams();
			
			//	Layout the content of the frame.
			Container cp = this.getContentPane();
			cp.setLayout(new BorderLayout());
			
			// Create an info region at the top.
			JPanel panel = layoutInfoRgn(aName, aVersion, "Written by:  " + aAuthor, aModDate);
			panel.setBorder(BorderFactory.createEmptyBorder(20,20,20,20));
			cp.add("North", panel);

			JPanel centerPanel = new JPanel(new BorderLayout());
			cp.add("Center", centerPanel);
			
			//  Load the illustration image into the center of the window.
			URL imgURL = SupersonicConeApp.class.getResource("images/illustration.gif");
			ImageIcon icon = new ImageIcon(imgURL);
			centerPanel.add("Center", new JLabel(icon));
			
			//	Display input boxes for all the parameters in this window.
			int i = 0;
			pLabels = new String[3];
			pLabels[i] = "Mach";	++i;
			pLabels[i] = "thetaC";	++i;
			pLabels[i] = "thetaS";	++i;
			
			//	Add the input parameters to be displayed to the window.
			panel = layoutOrderedParams(pdb, pLabels, true);
			centerPanel.add("North", panel);

			i = 0;
			pLabels = new String[7];
			pLabels[i] = "TT/T";		++i;
			pLabels[i] = "PT/P";		++i;
			pLabels[i] = "rhoT/rho";	++i;
			pLabels[i] = "Tc/T0";		++i;
			pLabels[i] = "Pc/P0";		++i;
			pLabels[i] = "rhoc/rho0";	++i;
			pLabels[i] = "Mc";			++i;

			//	Add the output parameters to be displayed to the window.
			panel = layoutOrderedParams(pdb, pLabels, false);
			centerPanel.add("South", panel);

			//  Size the window and then display it.
			AppUtilities.positionWindow( this, fWidth, fHeight );
		    this.setVisible(true);
			
			//  Add this window to the list of open windows maintained by MDIApplication.
			addWindow(this);
		}
		

		/**
		*  Creates and lays out a panel that contains the header information
		*  and "clear all" button.
		*
		*  @param  label1  The primary message to display in the info region.
		*  @param  label2  The secondary message to display in the info region.
		*  @return A panel containing the header information and clear all button.
		**/
		private JPanel layoutInfoRgn(String label1, String label2, String label3, String label4) {
			
			JPanel infoRgnPanel = new JPanel(new BorderLayout());
			
			// Place information labels.
			Box infoPanel1 = new Box(BoxLayout.Y_AXIS);
			JLabel infoLabel = new JLabel(label1);
			infoPanel1.add(infoLabel);
			infoPanel1.add(Box.createVerticalStrut(5));
			infoLabel = new JLabel(label2);
			infoPanel1.add(infoLabel);

			Box infoPanel2 = new Box(BoxLayout.Y_AXIS);
			infoLabel = new JLabel(label3);
			infoPanel2.add(infoLabel);
			infoPanel2.add(Box.createVerticalStrut(5));
			infoLabel = new JLabel(label4);
			infoPanel2.add(infoLabel);

			Box infoPanel3 = new Box(BoxLayout.X_AXIS);
			infoPanel3.add(infoPanel1);
			infoPanel3.add(Box.createHorizontalStrut(40));
			infoPanel3.add(infoPanel2);

			infoRgnPanel.add("West", infoPanel3);
			
			//	Place Clear button.
			JButton clearAllButton = new JButton("Clear All");
			clearAllButton.addActionListener(new ActionListener() {
					public void actionPerformed( ActionEvent evt ) {
						AutoParam.clearAllParams(pdb);
					}
				});
			clearAllButton.setToolTipText("Resets all the parameters in this window.");
			infoRgnPanel.add("East", clearAllButton);
			
			infoRgnPanel.setBorder(new EmptyBorder(10,20,10,20));
			
			return infoRgnPanel;
		}

		/**
		*  Lays out the selected parameters in the parameter database by
		*  adding them as components in a grid bag layout to a new panel
		*  and returning that panel.
		*
		*  @param  pdb      The parameter database being used by this
		*                   program.
		*  @param  pLabels  An list of string labels for the parameters that are
		*                   to be displayed.
		*  @param  enabled  Set to true to enable the text edit boxes for editing, false to disable them.
		*  @return A panel containing all the parameter input boxes.
		**/
		private JPanel layoutOrderedParams(ParamDatabase pdb, String[] pLabels, boolean enabled) throws MalformedURLException {
			int	i = 0;
			int columns = 3;
			AutoParam theParam;
			AutoParamInput	thePI=null;
			int xPos=0, yPos=0;
			
			//	Set up for grid layout.
			GridBagConstraints constraints = new GridBagConstraints();
			GridBagLayout gridbag = new GridBagLayout();
			JPanel panel = new JPanel(gridbag);
			
			//	Loop over elements in database, placing each one in this container.
			int length = pLabels.length;
			while ( i < length ) {
				AppUtilities.buildConstraints(constraints, xPos,yPos, 1,1, 100/columns, 10);
				constraints.fill = GridBagConstraints.BOTH;		//	How the component should fill cell.
				constraints.anchor = GridBagConstraints.CENTER;	//	Justification in cell
				
				//	Get the next element from database.
				theParam = (AutoParam)pdb.get(pLabels[i]);
				
				if (theParam == null)
					throw new NullPointerException("Parameter " + pLabels[i] + " not found in database.");
					
				//	Create text field for inputing this parameter's value.
				thePI = new AutoParamInput( theParam );
				thePI.setEditable(enabled);
				gridbag.setConstraints(thePI, constraints);
				panel.add(thePI);
				
				++xPos;
				if (xPos >= columns) {
					xPos = 0;
					++yPos;
				}
				
				++i;
			}
			
			return panel;
		}
			

		/**
		*  Initialize all the parameter objects used by this program.
		**/
		public void initParams() throws Exception {
			
			//  Create lists of units according to type.
			Unit[] angularUnits = {	Degrees.getInstance(), Radians.getInstance() };
			
			//	Mach number parameter
			// Create calculation methods.
			ParamMethod method = new MachMethod("thetaC", "thetaS");
			// Create the new parameter object.
			URL imgURL = SupersonicConeApp.class.getResource("images/Minf.gif");
			AutoParam aParam = new AutoParam("Mach", imgURL, null, method, pdb );
			aParam.setDescription("Free-stream Mach number.");
			aParam.setParamVerifier( (ParamVerifier)method );

			//  Cone half-angle paramter.
			method = new ThetaCMethod("Mach", "thetaS");
			imgURL = SupersonicConeApp.class.getResource("images/thetaC.gif");
			aParam = new AutoParam("thetaC", imgURL, Degrees.getInstance(), method, pdb );
			aParam.setAvailUnits(angularUnits);
			aParam.setDescription("Cone half-angle.");
			aParam.setParamVerifier( (ParamVerifier)method );
			
			//  Shock angle paramter.
			method = new ThetaSMethod("Mach", "thetaC");
			imgURL = SupersonicConeApp.class.getResource("images/thetaS.gif");
			aParam = new AutoParam("thetaS", imgURL, Degrees.getInstance(), method, pdb );
			aParam.setAvailUnits(angularUnits);
			aParam.setDescription("Oblique shock wave angle.");
			aParam.setParamVerifier( (ParamVerifier)method );
			
			
			//  Total temperature ratio paramter.
			method = new TT_TMethod("Mach", "thetaS");
			imgURL = SupersonicConeApp.class.getResource("images/TT_T.gif");
			aParam = new AutoParam("TT/T", imgURL, null, method, pdb );
			aParam.setDescription("Total-temp. to static temp. ratio at cone surface.");
			
			//  Total pressure ratio paramter.
			method = new PT_PMethod("Mach", "thetaS");
			imgURL = SupersonicConeApp.class.getResource("images/PT_P.gif");
			aParam = new AutoParam("PT/P", imgURL, null, method, pdb );
			aParam.setDescription("Total-pressure to static pressure ratio at cone surface.");
			
			//  Total density ratio paramter.
			method = new RhoT_RhoMethod("Mach", "thetaS");
			imgURL = SupersonicConeApp.class.getResource("images/rhoT_rho.gif");
			aParam = new AutoParam("rhoT/rho", imgURL, null, method, pdb );
			aParam.setDescription("Total-density to static density ratio at cone surface.");
			
			
			//  Ratio of static temperature at the cone surface to the free-
			//  stream static temperature paramter.
			method = new Tc_T0Method("Mach", "thetaS");
			imgURL = SupersonicConeApp.class.getResource("images/Tc_Tinf.gif");
			aParam = new AutoParam("Tc/T0", imgURL, null, method, pdb );
			aParam.setDescription("Static temp. at cone surface to free-stream static temp. ratio.");
			
			//  Ratio of static pressure at the cone surface to the free-
			//  stream static pressure paramter.
			method = new Pc_P0Method("Mach", "thetaS");
			imgURL = SupersonicConeApp.class.getResource("images/Pc_Pinf.gif");
			aParam = new AutoParam("Pc/P0", imgURL, null, method, pdb );
			aParam.setDescription("Static pressure at cone surface to free-stream static pressure ratio.");
			
			//  Ratio of static density at the cone surface to the free-
			//  stream static density paramter.
			method = new Rhoc_Rho0Method("Mach", "thetaS");
			imgURL = SupersonicConeApp.class.getResource("images/rhoC_rhoinf.gif");
			aParam = new AutoParam("rhoc/rho0", imgURL, null, method, pdb );
			aParam.setDescription("Static density at cone surface to free-stream static density ratio.");
			
			//  Mach number at the surface of the cone.
			method = new McMethod("Mach", "thetaS");
			imgURL = SupersonicConeApp.class.getResource("images/Mc.gif");
			aParam = new AutoParam("Mc", imgURL, null, method, pdb );
			aParam.setDescription("Mach number at the cone surface.");
			

			//	Have each parameter link up with the parameters that it depends on.
			AutoParam.linkAllParams(pdb);
		}
	}   //  end class AppWindow
	
}   //  end class SupersonicConeApp
