/**
*   Please feel free to use any fragment of the code in this file that you need
*   in your own work. As far as I am concerned, it's in the public domain. No
*   permission is necessary or required. Credit is always appreciated if you
*   use a large chunk or base a significant product on one of my examples,
*   but that's not required either.
*
*   This code is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
*      --- Joseph A. Huwaldt
**/

import java.awt.Component;
import javax.swing.JOptionPane;
import jahuwaldt.tools.ParamTools.*;


/**
*  Calculates the oblique conical shock angle based on free-stream
*  Mach number and the cone half-angle.
*
*  <p>  Modified by:  Joseph A. Huwaldt   </p>
*
*  @author   Joseph A. Huwaldt  Date: November 1, 2004
*  @version  November 7, 2004
**/
public class ThetaSMethod extends ParamMethod implements SharedData, ParamVerifier {
	
	/**
	*  Constructor for this method.
	*
	*  @param   Mach  Label for the "free-stream Mach number" parameter.
	*  @param   thetaC  Label for the "cone half-angle" parameter.
	**/
	public ThetaSMethod(String Mach, String thetaC) {
		inputLabels = new String[2];
		inputLabels[0] = Mach;
		inputLabels[1] = thetaC;
	}
	
	
	/**
	*  Calculates the oblique conical shock angle based on the free-stream
	*  Mach number and the cone half-angle.
	*
	*  @param   inputs  Array of input parameters.
	*                   [0=free-stream Mach number (Mach), 1=cone half-angle (thetaC)].
	*  @return  Oblique conical shock angle.
	**/
	public float calc(float[] inputs) throws Exception {
		float Mach = inputs[0];
		float thetaC = inputs[1];
		
		float thetaS = (float)conicalShock.shockAngle(Mach, thetaC, conicalShock.kWeak);
		
		return thetaS;
	}

	/**
	*  Check to see if the input value is valid for this parameter.
	*
	*  @param   root   The root component containing the parameter input text field
	*                  for the parameter being verified.
	*  @param   value  The value of the parameter as input by the user.
	*  @return  If "value" is valid, it should be output.  If "value" is invalid or
	*           out of range, a valid value is output or Float.NaN if nothing else.
	*           If ParamInput (which calls this routine) recieves a value of Float.NaN,
	*           it will clear the parameter.
	**/
	public float check(Component root, float value) {
		float outValue = value;
		
		if (outValue > (float)(Math.PI/2.0)) {
			outValue = (float)(Math.PI/2.0);
			
			JOptionPane.showMessageDialog ( root,
                                "The shock angle can not be greater than 90 degrees.",
								"Out of Range",
								JOptionPane.ERROR_MESSAGE );
		}
		
		return outValue;
	}

}
