/**
*   Please feel free to use any fragment of the code in this file that you need
*   in your own work. As far as I am concerned, it's in the public domain. No
*   permission is necessary or required. Credit is always appreciated if you
*   use a large chunk or base a significant product on one of my examples,
*   but that's not required either.
*
*   This code is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
*      --- Joseph A. Huwaldt
**/
package jahuwaldt.swing;

import java.lang.reflect.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.image.ImageObserver;
import java.net.URL;
import java.net.MalformedURLException;
import java.io.File;
import java.io.FilenameFilter;
import java.io.StringWriter;
import java.io.PrintWriter;
import javax.swing.*;
import java.util.List;
import java.util.ArrayList;
import java.util.Calendar;


/**
*  A set of generic utilities that I have found useful and that are used by
*  most of my Java applications.
*
*  <p>  Modified by:  Joseph A. Huwaldt   </p>
*
*  @author   Joseph A. Huwaldt   Date:  February 16, 2000
*  @version  September 18, 2008
**/
public class AppUtilities {

	// Used to neatly locate new windows on the display.
	private static int nthWindow = 0;

	//	OS flags.
	private static final boolean kIsMacOS = System.getProperty("mrj.version") != null;
	private static final boolean kIsWindows = System.getProperty("os.name").startsWith("Windows");
	
	
	/**
	*  Prevent anyone from instantiating this utiltity class.
	**/
	private AppUtilities() {}
	

	/**
	*  Returns true if this program is running in a MacOS 8/9 environment,
	*  false is returned otherwise.
	**/
	public static boolean isMacOSClassic() {
		return kIsMacOS && System.getProperty("os.name").equalsIgnoreCase("Mac OS");
		
	}

	/**
	*  Returns true if this program is running in a MacOS X environment,
	*  false is returned otherwise.
	**/
	public static boolean isMacOSX() {
		return kIsMacOS && System.getProperty("os.name").equalsIgnoreCase("Mac OS X");
	}

	/**
	*  Returns true if this program is running in any MacOS (8/9/X) environment,
	*  false is returned otherwise.
	**/
	public static boolean isMacOS() {
		return kIsMacOS;
	}

	/**
	*  Returns true if this program is running in a MS Windows environment,
	*  false is returned otherwise.
	**/
	public static boolean isWindows() {
		return kIsWindows;
	}

	/**
	*  Method that displays a dialog with a scrollable text field that
	*  contains the text of a Java exception message.  This allows caught exceptions to
	*  be displayed in a GUI rather than being hidden at a console window or not displayed
	*  at all.
	*
	*  @param parent  The component that the exception dialog should be associated with (null is fine).
	*  @param title   The title of the dialog window.
	*  @param message An optional message to display above the text pane (null is fine).
	*  @param th      The exception to be displayed in the text pane of the dialog.
	**/
    public static void showException(Component parent, String title, String message, Throwable th)  {
        StringWriter tracer = new StringWriter();
        th.printStackTrace(new PrintWriter(tracer, true));
        String trace = tracer.toString();
        JPanel view = new JPanel(new BorderLayout());
        if (message != null)
            view.add("North", new JLabel(message));
        view.add("Center", new JScrollPane(new JTextArea(trace, 10, 40)));
        JOptionPane.showMessageDialog(parent, view, title, 0);
    }


	/**
	*  Center the "inside" component inside of the bounding rectangle of
	*  the "outside" component.
	*
	*  @returns  A point representing the upper left corner location
	*            required to center the inside component in the
	*            outside one.
	**/
	public static Point centerIt( Component outside, Component inside ) {
		if (outside == null)	return centerIt(inside);
		Dimension outerSize = outside.getSize();
		Dimension innerSize = inside.getSize();
		Point outerLoc = outside.getLocationOnScreen();
		Point innerLoc = new Point();
		innerLoc.x = (outerSize.width - innerSize.width) / 2 + outerLoc.x;
		innerLoc.y = (outerSize.height - innerSize.height) / 2 + outerLoc.y;
		return innerLoc;
	}

	/**
	*  Center the specified component on the screen.
	*
	*  @returns  A point representing the upper left corner location
	*            required to center the specified component on the
	*            screen.
	**/
	public static Point centerIt( Component inComponent ) {
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension componentSize = inComponent.getSize();
		Point componentLoc = new Point( 0, 0 );
		componentLoc.x = (screenSize.width - componentSize.width) / 2;
		componentLoc.y = screenSize.height / 2 - componentSize.height / 2;
		return componentLoc;
	}

	/**
	*  Returns a point that can be used to locate a component in
	*  the "dialog" position (1/3 of the way from the top to the
	*  bottom of the screen).
	**/
	public static Point dialogPosition( Component inComponent ) {
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension componentSize = inComponent.getSize();
		Point componentLoc = new Point( 0, 0 );
		componentLoc.x = (screenSize.width - componentSize.width) / 2;
		componentLoc.y = screenSize.height / 3 - componentSize.height / 2;
		
		// Top of screen check
		componentLoc.y = (componentLoc.y > 0) ? componentLoc.y : 20;
		return componentLoc;
	}

	/**
	*  Positions the specified window neatly on the screen such
	*  that document windows are staggered one after the other.
	*  Care is also taken to make sure the window will fit on the screen.
	*
	*  @param  inWindow  The window to be positioned.
	*  @param  width     The width of the window.
	*  @param  height    The height of the window.
	**/
	public static void positionWindow( Window inWindow, int width, int height ) {
		final int margin = 30;
		
		// Window location: staggered
		int xOffset = (nthWindow * 20) + margin;
		int yOffset = (nthWindow * 20) + margin;
		Point windowLoc = new Point( xOffset, yOffset );

		// Window size: minimum of either the input dimensions or the display size.
		Dimension windowSize = inWindow.getToolkit().getScreenSize();
		windowSize.width -= xOffset + margin;
		windowSize.height -= yOffset + margin;
		if ( (width > 0) && (width < windowSize.width) )
			windowSize.width = width;
		
		if ( (height > 0) && (height < windowSize.height) )
			windowSize.height = height;

		// Set final size and location
		inWindow.setLocation( windowLoc );
		inWindow.setSize( windowSize );

		// Next window position
		nthWindow = (nthWindow < 5) ? nthWindow + 1 : 0;
	}

	/**
	*  Positions the specified window neatly on the screen such
	*  that document windows are staggered one after the other.
	*  This version assumes that windows are a fixed size and can
	*  not be shrunk.
	**/
	public static void positionWindowFixedSize( Window inWindow, int width, int height ) {
		final int margin = 30;
		
		// Window location
		int xOffset = (nthWindow * 20) + margin;
		int yOffset = (nthWindow * 20) + margin;
		Point windowLoc = new Point( xOffset, yOffset );

		// Window size
		Dimension screenSize = inWindow.getToolkit().getScreenSize();
		
		// Make sure fixed size window fits.
		int sum = xOffset + margin + width;
		if (sum > screenSize.width)
			windowLoc.x -= sum - screenSize.width;
		
		sum = yOffset + margin + height;
		if (sum > screenSize.height)
			windowLoc.y -= sum - screenSize.height;
		
		// Set final size and location
		inWindow.setLocation( windowLoc );
		inWindow.setSize( width, height );

		// Next window position
		nthWindow = (nthWindow < 5) ? nthWindow + 1 : 0;
	}

	/**
	*  Fills in the GridBagConstraints record for a given component
	*  with input items.
	*
	*  @param  gx,gy  Grid (cell) indicies for this compenent in the x and y dir
	*                 (1,3 == 2nd column, 4th row).
	*  @param  gw,gh  The number of cells this component spans.
	*  @param  wx,wy  Proportional width/height of this grid cell compared to others.
	**/
	public static void buildConstraints(GridBagConstraints gbc, int gx, int gy,
								int gw, int gh, int wx, int wy) {
		gbc.gridx = gx;
		gbc.gridy = gy;
		
		gbc.gridwidth = gw;
		gbc.gridheight = gh;
		
		gbc.weightx = wx;
		gbc.weighty = wy;
		
	}
	
	/**
	*  Try and determine the directory to where the program was installed and
	*  return that as a URL.  Has essentially the same function as
	*  Applet.getDocumentBase(), but works on local file systems
	*  in applications.
	**/
    public static URL getDocumentBase() {
	
		//	Object is to try and figure out where the program was installed.
		
		//	First see if the program was installed using ZeroG's Install Anywhere installer.
		String dir = System.getProperty( "lax.root.install.dir" );
		
		if (dir == null || dir.equals(""))
			//	Fall back on "user.dir" if all else fails.
			//	However, on some systems this will return the directory where the program is executed,
			//	not where it is installed.
			dir = System.getProperty( "user.dir" );
		
		//	Deal with different file separator characters.
		String urlDir = dir.replace( File.separatorChar, '/' );
		if (!urlDir.endsWith("/"))
			urlDir = urlDir + "/";
		
		URL output = null;
		try {
	    	output = new URL( "file", null, urlDir);
	    	
	    } catch ( MalformedURLException e ) { }
		
		return output;
	}
    

	/**
	*  Return the root component of the given component.
	*
	*  @param source  The Component or MenuComponent to find the root
	*                 Component for.
	**/
	public static Component getRootComponent(Object source) {
		Component root = null;
		
		if (source instanceof Component)
			root = SwingUtilities.getRoot((Component)source);
		
		else if (source instanceof MenuComponent) {
			MenuContainer mParent = ((MenuComponent)source).getParent();
			if (mParent instanceof Component)
				root = SwingUtilities.getRoot((Component)mParent);
		}
		
		return root;
	}
	
	
	/**
	*  Method that loads an image from a URL and creates a custom mouse cursor from it.
	*  The image is loaded using "Toolkit.getImage()".
	*  Requires Java 1.2 or later.
	*
	*  @param  url      A URL to the image to be loaded as a cursor.
	*  @param  hsx, hsy The point on the image that represents the cursor hot spot.
	*  @param  name     The name to assign to this cursor for Java Accessability.
	*  @param  observer The component to use to observe the loading of the image.  Pass null if none.
	*  @return The custom cursor generated from the specified image.  If any error occures
	*          null will be returned.
	**/
	public static Cursor getImageCursor(URL url, int hsx, int hsy, String name, ImageObserver observer) {
		Toolkit tk = Toolkit.getDefaultToolkit();
		Image img = tk.getImage(url);
		return makeImageCursor(img, hsx, hsy, name, observer);
	}
	
	/**
	*  Method that loads an image from a file and creates a custom mouse cursor from it.
	*  The image is loaded using "Toolkit.getImage()".
	*  Requires Java 1.2 or later.
	*
	*  @param  path     The path to the image to be loaded as a cursor.
	*  @param  hsx, hsy The point on the image that represents the cursor hot spot.
	*  @param  name     The name to assign to this cursor for Java Accessability.
	*  @param  observer The component to use to observe the loading of the image.  Pass null if none.
	*  @return The custom cursor generated from the specified image.  If any error occures
	*          null will be returned.
	**/
	public static Cursor getImageCursor(String path, int hsx, int hsy, String name, ImageObserver observer) {
		Toolkit tk = Toolkit.getDefaultToolkit();
		Image img = tk.getImage(path);
		return makeImageCursor(img, hsx, hsy, name, observer);
	}
	
	/**
	*  Method that creates a custom mouse cursor from the specified image.
	*  Requires Java 1.2 or later.
	*
	*  @param  img      The image to be loaded as a cursor.
	*  @param  hsx, hsy The point on the image that represents the cursor hot spot.
	*  @param  name     The name to assign to this cursor for Java Accessability.
	*  @param  observer The component to use to observe the loading of the image.  Pass null if none.
	*  @return The custom cursor generated from the specified image.  If any error occures
	*          null will be returned.
	**/
	public static Cursor makeImageCursor(Image img, int hsx, int hsy, String name, ImageObserver observer) {
		Cursor cursor = null;
		
		if (img == null)	return null;
		
		try {
			Toolkit tk = Toolkit.getDefaultToolkit();
		
			//	Wait for the image to load.
			int width = 0,  height = 0;
			int count = 0;
			while ((width < 1 || height < 1) && count < 10000) {
				width = img.getWidth(observer);
				height = img.getHeight(observer);
				++count;
			}
			
			if (width > 0 && height > 0) {
				//	Scale hot spot to the best cursor size.
				Dimension bestSize = tk.getBestCursorSize(width, height);
				
				if (bestSize.width > 0 && bestSize.height > 0) {
					Point hotSpot = new Point(hsx*bestSize.width/width, hsy*bestSize.height/height);
					
					//	Create the cursor.
					cursor = tk.createCustomCursor( img, hotSpot, name );
				}
			}
			
		} catch (IndexOutOfBoundsException e) {
			//	Just return null.
		}
		
		return cursor;
	}
	
	/**
	*  Sets the Swing look and feel to hide that hideous default Java LAF.
	**/
	public static void setSystemLAF() throws UnsupportedLookAndFeelException,
							IllegalAccessException, ClassNotFoundException, InstantiationException {
	
		// Set the system look and feel to hide that hideous Java LAF.
		String laf = UIManager.getSystemLookAndFeelClassName();
		UIManager.setLookAndFeel( laf );

	}
	
	
	/**
	*  Returns true if the current look and feel is the same as the system look and feel.
	*  Otherwise, returns false.
	**/
	public static boolean isSystemLAF() {
		return UIManager.getSystemLookAndFeelClassName().equals(UIManager.getLookAndFeel().getClass().getName());
	}
	
	
	/**
	*  <p>  Build up a JMenu from a description stored in a String
	*       array.  The description contains the text of each menu
	*       item, accelerator key, and the name of the method in the specified frame
	*       that handles the user choosing that menu item.
	*  </p>
	*  <p>  This is accomplished by the miracle of reflection.  </p>
	*
	*  @param  parent    This is the object that must contain the methods used to handle
	*                    action events for this menu's items.
	*  @param  name      Name of the menu to create.
	*  @param  menuDesc  String array that describes each of the menu items
	*          that will be built into this menu.  Item 0 is the menu item
	*          text string.  If this is null a menu seperator will be placed in
	*          the menu at this location.  Item 1 is the accelerator key for the menu.
	*          Item 2 is the name of the method in "parent" that will handle the user
	*          choosing this menu item. If this is null, the menu item will be shown as disabled.
	*  @returns A menu built up from the menu description supplied.
	**/
	public static JMenu buildMenu( Object parent, String name, String[][] menuDesc )
						throws NoSuchMethodException {
		if ( menuDesc == null || parent == null ) {
			throw new IllegalArgumentException( "menuDesc or parent are null!" );
		}

		// Get theis platform's menu accelerator key mask.
		Toolkit tk = Toolkit.getDefaultToolkit();
		int accCharMask = tk.getMenuShortcutKeyMask();
		
		// Create a menu to add items to.
		JMenu aMenu = new JMenu( name );

		// Add a list of items to a specified menu.

		int numItems = menuDesc.length;
		for ( int i = 0; i < numItems; ++i ) {
			String menuString = menuDesc[i][0];

			// Is this a menu seperator?
			if ( menuString == null ) {
				JSeparator separator = new JSeparator();
				aMenu.add( separator );
				
			} else {

				// Create the menu item.
				JMenuItem menuItem = new JMenuItem( menuString );

				// Does this menu item have an accellerator key?
				String accelerator = menuDesc[i][1];
				if ( accelerator != null && !accelerator.equals( "" ) ) {
					// Add the accelerator key.
					menuItem.setAccelerator( KeyStroke.getKeyStroke( accelerator.charAt( 0 ), accCharMask, false ) );
				}

				// Does this menu item have an action method?
				String methodStr = menuDesc[i][2];
				if ( methodStr != null && !methodStr.equals( "" ) ) {
					// Create an action listener by reflection that refers back to
					// the specified method in the specified frame.
					ActionListener listener = getActionListenerForMethod(parent, methodStr);
					menuItem.addActionListener( listener );
					menuItem.setActionCommand( menuString );
					menuItem.setEnabled( true );
					
				} else {
					// If no method, disable this menu item.
					menuItem.setEnabled( false );
				}

				// Add this new menu item to the specified menu.
				aMenu.add( menuItem );
			}
		}

		return aMenu;
	}

	/**
	*  <p>  Build up a ButtonGroup containing toggle buttons from a description
	*       stored in a String array.  The description contains, for each button,
	*       the text to appear in the button (null if no text), the name of
	*       the method in the parent object that will be called when a button is
	*       clicked on (if null is passed, the button is disabled), and the tool tip
	*       text to display for the button (null for no tool tip).
	*  </p>
	*  <p>  This is accomplished by the miracle of reflection.  </p>
	*
	*  @param  parent    The parent object that will contain handle the user clicking
	*                    on one of the buttons in this group.  This object is also
	*                    used to observe the reading in of the button icon images.
	*  @param  defs      String array that describes each of the text string that will
	*          be displayed in the button.  Item 0 is the text to be displayed in each
	*          button.  If this is null no text is displayed.
	*          Item 1 is the name of the method in "parent" that will handle the user
	*          clicking on this button item. If this is null, the menu button will be
	*          shown as disabled.  Item 2 is the tool tip to be shown above this button
	*          item.  If null is passed no tool tip is shown.
	*  @param  imgURLs   An array of URLs for the images to be displayed in the buttons.
	*                    If "null" is passed in, then there will be no button images.
	*                    If any element is null, that button will have no image.
	*  @returns A button group built up from the button description supplied.
	**/
	public static ButtonGroup buildButtonGroup( ImageObserver parent, String[][] defs, URL[] imgURLs ) {
		
		ImageIcon[] icons = null;
		
		if (imgURLs != null) {
			int length = imgURLs.length;
			if (defs.length != length)
				throw new IllegalArgumentException("The defs & imgURLs arrays have different sizes.");
			icons = new ImageIcon[length];
			for (int i=0; i < length; ++i) {
				if (imgURLs[i] != null)
					icons[i] = new ImageIcon(imgURLs[i]);
			}
		}
		
		return buildButtonGroup(parent, defs, icons);
	}
	
	/**
	*  <p>  Build up a ButtonGroup containing toggle buttons from a description
	*       stored in a String array.  The description contains, for each button,
	*       the text to appear in the button (null if no text), the name of
	*       the method in the parent object that will be called when a button is
	*       clicked on (if null is passed, the button is disabled), and the tool tip
	*       text to display for the button (null for no tool tip).
	*  </p>
	*  <p>  This is accomplished by the miracle of reflection.  </p>
	*
	*  @param  parent    The parent object that will contain handle the user clicking
	*                    on one of the buttons in this group.  This object is also
	*                    used to observe the reading in of the button icon images.
	*  @param  defs      String array that describes each of the text string that will
	*          be displayed in the button.  Item 0 is the text to be displayed in each
	*          button.  If this is null no text is displayed.
	*          Item 1 is the name of the method in "parent" that will handle the user
	*          clicking on this button item. If this is null, the menu button will be
	*          shown as disabled.  Item 2 is the tool tip to be shown above this button
	*          item.  If null is passed no tool tip is shown.
	*  @param  imgPaths  An array of image file paths for the images to be displayed in the buttons.
	*                    If "null" is passed in, then there will be no button images.
	*                    If any element is null, that button will have no image.
	*  @returns A button group built up from the button description supplied.
	**/
	public static ButtonGroup buildButtonGroup( ImageObserver parent, String[][] defs, String[] imgPaths ) {
		
		ImageIcon[] icons = null;
		
		if (imgPaths != null) {
			int length = imgPaths.length;
			if (defs.length != length)
				throw new IllegalArgumentException("The defs & imgPaths arrays have different sizes.");
			icons = new ImageIcon[length];
			for (int i=0; i < length; ++i) {
				if (imgPaths[i] != null)
					icons[i] = new ImageIcon(imgPaths[i]);
			}
		}
		
		return buildButtonGroup(parent, defs, icons);
	}
	
	/**
	*  <p>  Build up a ButtonGroup containing toggle buttons from a description
	*       stored in a String array.  The description contains, for each button,
	*       the text to appear in the button (null if no text), the path to an
	*       image file to display in the button (null for no image), the name of
	*       the method in the parent object that will be called when a button is
	*       clicked on (if null is passed, the button is disabled), and the tool tip
	*       text to display for the button (null for no tool tip).
	*  </p>
	*  <p>  This is accomplished by the miracle of reflection.  </p>
	*
	*  @param  parent    The parent object that will contain handle the user clicking
	*                    on one of the buttons in this group.  This object is also
	*                    used to observe the reading in of the button icon images.
	*  @param  defs      String array that describes each of the text string that will
	*          be displayed in the button.  Item 0 is the text to be displayed in each
	*          button.  If this is null no text is displayed.
	*          Item 1 is the name of the method in "parent" that will handle the user
	*          clicking on this button item. If this is null, the menu button will be
	*          shown as disabled.  Item 2 is the tool tip to be shown above this button
	*          item.  If null is passed no tool tip is shown.
	*  @param  icons     An array of icon images to be displayed in the buttons.  If this
	*                    array is null, there will be no images displayed in the buttons.
	*                    If one of the elements of the array is null, that button will have
	*                    no image.
	*  @returns A button group built up from the button description supplied.
	**/
	public static ButtonGroup buildButtonGroup( ImageObserver parent, String[][] defs, ImageIcon[] icons ) {
	
		if (icons != null && defs.length != icons.length)
			throw new IllegalArgumentException("The defs & icons arrays have different sizes.");
			
		// Create a group for these buttons so that they toggle.
		ButtonGroup bGroup = new ButtonGroup();

		// Create buttons.
		int numBtns = defs.length;
		for ( int i = 0; i < numBtns; ++i ) {

			// Extract information about this button.
			String title = defs[i][0];
			String methodStr = defs[i][1];
			String toolTip = defs[i][2];
			ImageIcon icon = null;
			if (icons != null)	icon = icons[i];

			JToggleButton button = null;
			if ( title != null && !title.equals( "" ) ) {
				if ( icon != null )
					button = new JToggleButton( title, icon );
				else
					button = new JToggleButton( title );
				
			} else {
				if ( icon != null ) {
					button = new JToggleButton( icon );
					Image image = icon.getImage();
					Dimension size = new Dimension( image.getWidth( parent ),
														image.getHeight( parent ) );
					button.setMaximumSize( size );
				}
			}

			if ( button != null ) {
				bGroup.add( button );
				button.setAlignmentX( JToggleButton.CENTER_ALIGNMENT );
				button.setAlignmentY( JToggleButton.CENTER_ALIGNMENT );
				button.setMargin( new Insets( 0, 0, 0, 0 ) );
				if (toolTip != null)
					button.setToolTipText(toolTip);
				
				if ( i == 0 )
					button.setSelected( true );

				if ( methodStr != null && ! methodStr.equals( "" ) ) {
					try  {

						// Create an action listener by reflection that refers back to
						// the specified method in the specified frame.
						ActionListener listener = getActionListenerForMethod(parent, methodStr);
						button.addActionListener( listener );
						button.setActionCommand( title );
						button.setEnabled( true );
						
					} catch( NoSuchMethodException e ) {
						System.err.println( "Button action listener method not found!" );
						e.printStackTrace();
					}
					
				} else {
					// If no method, disable this menu item.
					button.setEnabled( false );
				}

			}
		}

		return bGroup;
	}

	/**
	*  <p>  Build up a List of buttons from a description stored in a String
	*       array.  The description contains, for each button, the text to appear in
	*       the button (null if no text), the path to an image file to display
	*       in the button (null for no image), the name of the method in
	*       the parent object that will be called when a button is clicked
	*       on (if null is passed, the button is disabled), and the tool tip
	*       text to display for the button (null for no tool tip).
	*  </p>
	*  <p>  This is accomplished by the miracle of reflection.  </p>
	*
	*  @param  parent    The parent object that will contain handle the user clicking
	*                    on one of the buttons in this group.  This object is also
	*                    used to observe the reading in of the button icon images.
	*  @param  defs      String array that describes each of the text string that will
	*          be displayed in the button.  If this is null no text is displayed.
	*          Item 1 is the file path to the image to display in this button.
	*          If null is passed, no image icon is displayed.  Item 2 is the name of the
	*          method in "parent" that will handle the user clicking on this button item.
	*          If this is null, the menu button will be shown as disabled.  Item 3
	*          is the tool tip to be shown above this button item.  If null is passed
	*          no tool tip is shown.
	*  @returns A List containing all the buttons built up from the description supplied.
	**/
	public static List buildButtonList( ImageObserver parent, String[][] defs ) {
		// Create a group for these buttons so that they toggle.
		List bList = new ArrayList();

		// Create buttons.
		int numBtns = defs.length;
		for ( int i = 0; i < numBtns; ++i ) {

			// Extract information about this button.
			String title = defs[i][0];
			String imagePath = defs[i][1];
			String methodStr = defs[i][2];
			String toolTip = defs[i][3];

			JButton button = null;
			if ( title != null && !title.equals( "" ) ) {
				if ( imagePath != null && !imagePath.equals( "" ) )
					button = new JButton( title, new ImageIcon( imagePath ) );
				else
					button = new JButton( title );
				
			} else {
				if ( imagePath != null && !imagePath.equals( "" ) ) {
					ImageIcon icon = new ImageIcon( imagePath );
					button = new JButton( icon );
					Image image = icon.getImage();
					Dimension size = new Dimension( image.getWidth( parent ),
														image.getHeight( parent ) );
					button.setMaximumSize( size );
				}
			}

			if ( button != null ) {
				bList.add( button );
				button.setAlignmentX( JButton.CENTER_ALIGNMENT );
				button.setAlignmentY( JButton.CENTER_ALIGNMENT );
				button.setMargin( new Insets( 0, 0, 0, 0 ) );
				if (toolTip != null)
					button.setToolTipText(toolTip);
				
				if ( methodStr != null && ! methodStr.equals( "" ) ) {
					try  {

						// Create an action listener by reflection that refers back to
						// the specified method in the specified frame.
						ActionListener listener = getActionListenerForMethod(parent, methodStr);
						button.addActionListener( listener );
						button.setActionCommand( title );
						button.setEnabled( true );

					} catch( NoSuchMethodException e ) {
						System.err.println( "Button action listener method not found!" );
						e.printStackTrace();
					}
					
				} else {
					// If no method, disable this menu item.
					button.setEnabled( false );
				}

			}
		}

		return bList;
	}

	
	/**
	*  Method that returns an action listener that simply calls the specified method in the
	*  specified class.  This is little trick is done using the magic of reflection.
	*
	*  @param  target     The object which contains the specified method.
	*  @param  methodStr  The name of the method to be called in the target object.
	*                     This method must be contained in target, must be publically
	*                     accessable and must accept an ActionEvent object as the
	*                     only parameter.
	*  @return An ActionListener that will call the specified method in the specified
	*          target object and pass to it an ActionEvent object.
	*  @throws NoSuchMethodException if the target object does not contain the specified
	*          method.
	**/
	public static ActionListener getActionListenerForMethod( Object target, String methodStr)
										throws NoSuchMethodException {
		Method m = target.getClass().getMethod( methodStr, new Class[] { ActionEvent.class } );
		ActionListener listener = new GenericActionListener( target, m );
		return listener;
	}


	/**
	*  Method that exits the program (with a warning message) if the current data is after
	*  the specified date.
	**/
	public static void checkDateAndDie(Calendar date, String message) {
		if (Calendar.getInstance().after(date)) {
		    JOptionPane.showMessageDialog( null, message, "Expired", JOptionPane.ERROR_MESSAGE );
			System.exit(0);
		}
	}


	/**
	*  Method that returns the index to an option that a user has selected from an array of options
	*  using a standard input dialog.
	*
	*  @param parent      The parent Component for the dialog
	*  @param msg         The message to display to the user.
	*  @param title       The title of the dialog window.
	*  @param messageType One of the JOptionPane message type constants: ERROR_MESSAGE,
	*                     INFORMATION_MESSAGE, WARNING_MESSAGE, QUESTION_MESSAGE, or PLAIN_MESSAGE.
	*  @param selectionValues       An array of Objects that gives the possible selections.
	*  @param initialSelectionValue The value used to initialize the input field.
	*  @return The index into the input array of the value the user has selected or -1 if the user
	*          canceled.
	**/
	public static int userSelectionFromArray(Component parent, Object msg, String title, int messageType,
							Object[] selectionValues, Object initialSelectionValue) {
		
		Object selection = JOptionPane.showInputDialog(parent, msg, title, messageType, null,
											selectionValues, initialSelectionValue);
		if (selection == null)	return -1;
		
		int length = selectionValues.length;
		int i=0;
		for (; i < length; ++i)
			if (selection.equals(selectionValues[i]))
				break;
				
		return i;
	}
	
	/**
	*  Method that returns the bounded integer value that a user has entered in a standard input dialog.
	*
	*  @param parent       The parent Component for the dialog
	*  @param msg          The message to display to the user.
	*  @param title        The title of the dialog window.
	*  @param messageType  One of the JOptionPane message type constants: ERROR_MESSAGE,
	*                      INFORMATION_MESSAGE, WARNING_MESSAGE, QUESTION_MESSAGE, or PLAIN_MESSAGE.
	*  @param initialValue The initial value for the dialog's text field.
	*  @param lowerBound   The lowest (most negative) value to allow the user to enter (use Integer.MIN_VALUE to
	*                      allow any negative integer).
	*  @param upperBound   The largest (most positive) value to allow the user to enter (use Integer.MAX_VALUE to
	*                      allow any positive integer).
	*  @return The integer value the user has entered or null if the user has canceled.
	**/
	public static Integer userEnteredInteger(Component parent, String msg, String title, int messageType,
							int initialValue, int lowerBound, int upperBound) {
		Integer value = null;
		String output = null;
		do {
			output = (String)JOptionPane.showInputDialog(parent, msg, title, messageType, null,
											null, new Integer(initialValue));
			if (output != null) {
				try {
					value = Integer.valueOf(output);
					if (value.intValue() < lowerBound || value.intValue() > upperBound)
						throw new NumberFormatException();
					break;
				} catch (NumberFormatException e) {
					Toolkit.getDefaultToolkit().beep();
				}
			}
		} while (output != null);
		
		return value;
	}

	/**
	*  Method that displays a "Save As..." dialog asking the user to select or input a file name to
	*  save a file to and returns a reference to the chosen file.  N.B.:  No test is made to see if the
	*  user can actually write to that file!  See:  File.canWrite().
	*
	*  @param parent    The owner of the dialog (null is fine).
	*  @param message   The message for the dialog.  Something like "Choose a name for this file:".
	*  @param directory The directory to prompt the user with by default (null is fine).
	*  @param name      The name of the file to propmt the user with by default (null is fine).
	*  @param filter    The filename filter to use (null means no filter).
	*  @param extension The filename extension.  This is appended to the filename provided by the user
	*                   if the filename input doesn't already have it.  Passing null means that no extension
	*                   will be forced onto the filename.
	*  @param existsErrMsg1  The 1st part of the "file exists" error message.  Pass something like,
	*						 "A file with the name \".
	*  @param existsErrMsg2  The 2nd part of the "file exists" error message.  Pass something like,
	*						 ""\" already exists.  Do you want to replace it?"
	*  @return The file chosen by the user for saving out a file.  Returns null if the a valid file was not
	*          chosen.
	**/
	public static File selectFile4Save(Frame parent, String message, String directory, String name,
					FilenameFilter filter, String extension, String existsErrMsg1, String existsErrMsg2) {
		File chosenFile = null;
		
		if (parent == null)
			parent = new Frame();
		if (message == null)
			message = "";
		
		// Bring up a file chooser.
		FileDialog fd = new FileDialog( parent, message, FileDialog.SAVE );
		
		if ( directory != null )
			// Prompt the user with the existing directory path.
			fd.setDirectory( directory );
		
		if ( name != null)
			//	Prompt the user with the existing file name.
			fd.setFile( name );
		
		if (filter != null)
			fd.setFilenameFilter( filter );
		fd.setVisible(true);
		
		// Get user input to the file dialog.
		String fileName = fd.getFile();
		if ( fileName != null ) {
		
			//	Create the file reference to the chosen file.
			chosenFile = new File( fd.getDirectory(), fileName );
				
			// The user has supplied a file name/path, make sure it has the required extension.
			if (extension != null) {
				if (!extension.startsWith("."))
					extension = "." + extension;
				
				if ( !fileName.toLowerCase().endsWith(extension) ) {
					//	Create a new file reference including the extension.
					chosenFile = new File( fd.getDirectory(), fileName + extension);
		
					//	Since we have changed the name from what the user input, make sure the newly named file
					//	doesn't already exist.
					if (chosenFile.exists()) {
						// Build up a message to show the user.
						StringBuffer buffer = new StringBuffer( existsErrMsg1 );
						buffer.append( chosenFile.getName() );
						buffer.append( existsErrMsg2 );
						
						//	Ask the user if they want to replace the existing file.
						int result = JOptionPane.showConfirmDialog( parent, buffer.toString(), "Warning",
												JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE );
						if ( result != JOptionPane.YES_OPTION )
							// The user does not want to overwrite the existing file.
							chosenFile = null;
					}
				}
			}
		}

		return chosenFile;
	}
}


