/**
*   Please feel free to use any fragment of the code in this file that you need
*   in your own work. As far as I am concerned, it's in the public domain. No
*   permission is necessary or required. Credit is always appreciated if you
*   use a large chunk or base a significant product on one of my examples,
*   but that's not required either.
*
*   This code is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
*      --- Joseph A. Huwaldt
**/
package jahuwaldt.swing;

import java.lang.ref.SoftReference;
import java.awt.Frame;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Stack;
import java.util.List;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.ResourceBundle;
import javax.swing.SwingUtilities;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import net.roydesign.app.Application;
import net.roydesign.app.AboutJMenuItem;


/**
*  This is a wrapper for an application instance.  It is an extension of
*  Steve Roy's excellent MRJ Adapter package "Application" that
*  adds Window Menu support and other features used by Multi-Document-Interface
*  (MDI) applications.  Under MacOS X, an application instance will automatically call
*  "handleNew()" method if the user brings the application forward and there
*  are no open window's registered.  On non-Mac platforms, the application will quit
*  once the last document window has been closed.
*
*  <p>  Modified by:  Joseph A. Huwaldt    </p>
*
*  @author    Joseph A. Huwaldt    Date:  April 25, 2004
*  @version   September 14, 2008
**/
public class MDIApplication extends Application {

	/**
	*  A list of all open document windows.
	**/
	private static Stack openWindows = new Stack();
	
	/**
	*  A list of references to JMenu objects used to hold "Windows" menus.
	**/
	private static List windowsMenus = new ArrayList();
	
	/**
	*  A list of objects that want to be notified if the application is going'
	*  to quit.
	**/
	private static List quitList = new ArrayList();
	
	/**
	*  The resource bundle for this application.
	**/
	private ResourceBundle resBundle = null;
	
	
	//-------------------------------------------------------------------------
	/**
	*  Constructor a new Application instance that has the specified name.
	*
	*  @param  name  The name of the application.
	**/
	public MDIApplication( String name ) {

		//  Set the application name.
		setName(name);
		
		//  Handle "Application ReOpen Event" by bringing a window forward or by creating a
		//  new blank document window if there are no currently open windows.
		addReopenApplicationListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// If a window exists, bring it to the front, otherwise create a new document window.
				if (!openWindows.empty()) {
					Frame window = (Frame)openWindows.peek();
					window.setVisible(true);
					
				} else {
					//  Create a new blank document window for this application.
					handleNew(null);
				}
			}
		});
	}

	//-------------------------------------------------------------------------
	/**
	*  Used to set the resource bundle for this application.  This is provided as
	*  a convencience for storing the resource bundle so that it is accessible
	*  throughout an application.
	*
	*  @param bundle  The resource bundle to be stored with this application.
	**/
	public void setResourceBundle(ResourceBundle bundle) {
		this.resBundle = bundle;
	}
	
	/**
	*  Returns the resource bundle stored with this application.  If no resource
	*  bundle has been stored, then null is returned.
	**/
	public ResourceBundle getResourceBundle() {
		return resBundle;
	}
	
	/**
	*  Register the supplied window with the list of windows managed by this
	*  MDI Application.  Technically, the window should be made visible before
	*  calling this method to avoid an inconsistant user interface state (a window
	*  listed in the "Windows" menu, but not visible).  The window will be removed
	*  from the "Windows" menu when it's "dispose" method is called.
	*
	*  @param  window  The window to be added to the list of open windows.
	**/
	public void addWindow(Frame window) {
		//  Add the window to the list of open windows.
		openWindows.push(window);
		
		//  Add this window to the Windows menus.
		addNewMenuItem(window);
		
		//  Add a window event listener in order to keep the open windows list and menus up to date.
		window.addWindowListener(new WindowAdapter() {
			public void windowClosed(WindowEvent e) {
				Object aWindow = e.getWindow();
				int idx = openWindows.indexOf(aWindow);
				removeMenuItem(openWindows.size() - idx - 1);
				openWindows.remove(idx);
				
				//  On on-Mac systems, quit if there are no open windows.
				if (!AppUtilities.isMacOS() && openWindows.size() == 0)
					handleQuit(null);
			}
			
			public void windowActivated(WindowEvent e) {
				Frame aWindow = (Frame)e.getWindow();
				int idx = openWindows.indexOf(aWindow);
				if (idx < 0)	return;
				removeMenuItem(openWindows.size() - idx - 1);
				openWindows.remove(idx);
				openWindows.push(aWindow);
				addNewMenuItem(aWindow);
			}
		});
		
	}
	
	/**
	*  Register an object to recieve notification that the application is going to quit soon.
	**/
	public void addQuitListener(QuitListener listener) {
		quitList.add(listener);
	}
	
	/**
	*  Method to remove a quit listener from the list of quit listeners for this application.
	**/
	public void removeQuitListener(QuitListener listener) {
		quitList.remove(listener);
	}
	
	/**
	*  Returns a reference to the top-most window in the list of all open windows registered with
	*  this application.  If there are no open windows, null is returned.
	**/
	public Frame getTopWindow() {
		Frame window = null;
		if (!openWindows.empty())
			window = (Frame)openWindows.peek();
		return window;
	}
	
	/**
	*  Get an unmodifiable list of all the currently open windows in the application.
	**/
	public List allOpenWindows() {
		return Collections.unmodifiableList(openWindows);
	}
	
	/**
	*  Get a new JMenu instance that can be used as a Windows menu using
	*  the specified title (typically "Windows").  The menu that is returned
	*  is maintained by this class and should not be modified by the user.
	**/
	public JMenu newWindowsMenu(String title) {
		JMenu menu = new JMenu(title);
		
		//  Add the new menu to our list of Windows menus.
		windowsMenus.add(new SoftReference(menu));
		
		createNewMenuItems(menu);
		
		return menu;
	}
	
	/**
	*  Call this method when a window being shown in a Windows menu has changed
	*  it's title.  This method will update the titles shown in the Windows menu.
	*
	*  @param  window  The window that has changed titles.
	**/
	public void windowTitleChanged(Frame window) {
		//  Find the window in the list of open windows.
		int idx = openWindows.indexOf(window);
		
		//  Adjust the index to be top down instead of bottom up.
		idx = openWindows.size() - idx - 1;
		
		if (idx >= 0 && windowsMenus.size() > 0 ) {
			//  Get the new title.
			String title = window.getTitle();
			
			//  Loop over all the Windows Menus in existance.
			for (Iterator i=windowsMenus.iterator(); i.hasNext(); ) {
				SoftReference ref = (SoftReference)i.next();
				JMenu menu = (JMenu)ref.get();
				
				if (menu != null) {
					JMenuItem item = menu.getItem(idx);
					item.setText(title);
				}
			}
		}
	}
	
	/**
	*  Method that adds a menu item to the specified menu for each
	*  open window.
	**/
	private void createNewMenuItems(JMenu menu) {
		for (int i=openWindows.size()-1; i >= 0; --i) {
			Frame window = (Frame)openWindows.get(i);
			JMenuItem item = new JMenuItem(window.getTitle());
			item.addActionListener(new MenuListener(window));
			menu.add(item);
		}
	}
	
	/**
	*  Method that adds a single new menu item to the top of ALL the Windows menus.
	**/
	private void addNewMenuItem(Frame window) {
		if (windowsMenus.size() > 0) {
		
			String title = window.getTitle();
			for (Iterator i=windowsMenus.iterator(); i.hasNext();) {
				SoftReference ref = (SoftReference)i.next();
				JMenu menu = (JMenu)ref.get();
				
				if (menu != null) {
					JMenuItem item = new JMenuItem(title);
					item.addActionListener(new MenuListener(window));
					menu.add(item, 0);
				} else {
					i.remove();
				}
			}
			
		}
	}
	
	/**
	*  Method that removes a single menu item from ALL of the Windows menus.
	**/
	private static void removeMenuItem(int idx) {
		if (windowsMenus.size() > 0) {
		
			for (Iterator i=windowsMenus.iterator(); i.hasNext();) {
				SoftReference ref = (SoftReference)i.next();
				JMenu menu = (JMenu)ref.get();
				
				if (menu != null) {
					menu.remove(idx);
				} else {
					i.remove();
				}
			}
			
		}
	}
	
	/**
	*  Class that provides an action listener for Window menu items.
	*  When this listener is called, it brings it's window to the front.
	**/
	private class MenuListener implements ActionListener {
		private final Frame theWindow;
		MenuListener(Frame window) {
			theWindow = window;
		}
		public void actionPerformed( ActionEvent event ) {
			theWindow.setVisible(true);
		}
	}
	
	/**
	*  Method used to create an about menu item for use in this application.
	*
	*  @return A reference to the newly created about menu item or null if an item was not created.
	**/
	public AboutJMenuItem createAboutMenuItem() {	return null;	}

	/**
	*  Handle the user requesting a new document window.
	*  If it is inappropriate for the application to create a new document
	*  window, then this method should do nothing.  The default
	*  implementation does nothing and returns null.
	*
	*  @param  event  The event that caused this method to be called.  May be "null"
	*                 if this method is called by MDIApplication or one of it's subclasses.
	*  @return A reference to the newly created window frame or null if a window was not created.
	**/
	public Frame handleNew( ActionEvent event) { return null; }
	
	/**
	*  Handle the user choosing "Close" from the File menu.  This implementation
	*  dispatches a "Window Closing" event on the top most window.
	**/
	public void handleClose( ActionEvent event ) {
		if (openWindows.size() > 0) {
			Frame window = (Frame)openWindows.peek();
			window.dispatchEvent( new WindowEvent( window, WindowEvent.WINDOW_CLOSING ) );
		}
	}

	private boolean quitFlag = false;
	
	/**
	*  Handle the user choosing "Quit" from the File menu.  This implementation
	*  safely works around a bug in Java on early versions of MacOS X that could
	*  cause a hang when displaying dialogs after choosing quit.  This implementation
	*  dispatches a seperate "window closing" event to each window.  This gives each
	*  window an opportunity to cancel the quit process.
	**/
	public synchronized void handleQuit( ActionEvent event ) {
		
		if (quitFlag)   return;
		
		if (quitList.size() > 0) {
			quitFlag = true;
			
			//	Start a new thread to cycle through all the quit listeners and tell them a quit is pending.
			//	The separate thread is required to prevent a hang in MacOS X.
			Runnable runme = new Runnable() {
				public void run() {
		
					// Loop over all the quit listeners, notifying them one at a time.
					// When the last notification is made.  The application will quit.
					int length = quitList.size();
					QuitListener[] array = new QuitListener[length];
					quitList.toArray(array);
					for (int i=0; i < length; ++i) {
						QuitListener next = array[i];
						
						//  Tell the listener that we are going to quit.
						boolean cancel = next.quit();
						if (cancel) {
							quitFlag = false;
							return;
						}
					}
					
					//  Once all the listeners have been notified, kill the JVM to quit the program.
					System.exit(0);
				}
			};
			//	Run this thread on the Swing event dispach thread to prevent hangs.
			SwingUtilities.invokeLater(runme);
			
		} else {
			//  There are no quit listeners, so just kill the JVM to quit the program.
			System.exit(0);
		}
		
	}
	
}
