/*
*   AdvParamInput - Object used for entering AdvParam values using a text field.
*
*   Copyright (C) 1998-2004 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Lesser General Public License for more details.
*
*   You should have received a copy of the GNU Lesser General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*   Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.tools.ParamTools;

import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.net.URL;
import java.net.MalformedURLException;
import java.text.NumberFormat;
import java.text.DecimalFormat;
import javax.swing.*;

import jahuwaldt.util.HelpSystem;
import jahuwaldt.tools.units.Unit;
import jahuwaldt.swing.DecimalField;


/**
*  Defines an Swing JPanel object that is used for entering
*  AdvParam type parameter values (using a text field).
*  Provides a GUI user interface to advanced parameter type
*  objects.
*
*  <p>  Modified by:  Joseph A. Huwaldt    </p>
*
*  @author  Joseph A. Huwaldt   Date: July 14, 1999
*  @version November 6, 2004
**/
public class AdvParamInput extends ParamInput {

	/**
	*  Panel for special controls at bottom of this panel.
	**/
	protected JPanel bottomPanel;

	//-----------------------------------------------------------------------------
	/**
	*  Don't allow the user to access the default constructor.
	**/
	protected AdvParamInput() { }

	/**
	*  Constructs an AdvParamInput class that accepts an AdvParam for input.
	*
	*  @param  The AdvParam object that this input field should use.
	*  @throws MalformedURLException if either the path to the symbol or help icon image
	*          files could not be combined with the application location and turned into a URL.
	**/
	public AdvParamInput( AdvParam param ) throws MalformedURLException {

		theParam = param;
		
		//	Create layout
		setLayout( new BorderLayout() );
		
		//	Create text input field for this parameter.
		nf = param.getNumberFormat();
		if (nf == null)
			nf = (DecimalFormat)NumberFormat.getInstance();

		if ( param.isDefined() )
			theTField = new DecimalField( param.getValue(), 10, nf );
		else
			theTField = new DecimalField( 10, nf );
			
		tfActionListener = new TextFieldListener();
		tfFocusListener = new TextFieldFocusL();
		theTField.addActionListener( tfActionListener );
		theTField.addFocusListener( tfFocusListener );
		add( "Center", theTField );
		
		
		//	Create the label for this parameter.
		URL symbolURL = param.getSymbolURL();
		JLabel pLabel = null;
		if ( symbolURL == null ) {
			//	No graphic symbol, use text label.
			pLabel = new JLabel( param.getLabel() + " =", JLabel.RIGHT );
			pLabel.setFont(new Font("Serif", Font.PLAIN, 12));
			
		} else {
			Icon icon = new ImageIcon(symbolURL);
			
			//	Graphic symbol provided, so use that.
			pLabel = new JLabel( icon, JLabel.RIGHT );
		}
		
		pLabel.setLabelFor(theTField);
		pLabel.setToolTipText(param.getDescription());
		add( "West", pLabel );
		
		
		//	Create the units label or choice menu for this parameter.
		if ( param.availUnits != null && param.availUnits.length > 1 ) {
		
			//	Many units are available.
			JComboBox unitList = new JComboBox(param.getAvailUnits());
			unitList.addActionListener(new UnitMenuListener());
			unitList.setToolTipText("Used to change parameter units.");
			
			//	Select the current units.
			Unit currentUnit = param.getUnits();
			unitList.setSelectedItem( currentUnit );
			add( "East", unitList );
			
		} else {
			//	Only 1 unit available.
			String unitStr = " ";
			Unit theUnits = param.getUnits();
			if ( theUnits != null ) {
				unitStr += theUnits.toString();
			}
			unitStr += " ";
			JLabel theLabel = new JLabel( unitStr, JLabel.LEFT );
			add( "East", theLabel );
		}
		
		
		//	Add a panel to the bottom for special controls.
		bottomPanel = new JPanel();
		add( "South", bottomPanel );
		
		//	Add a help button to the bottom panel (if help is available).
		if ( param.isHelpAvail() ) {
		
			//	If help button images exist, create an ImageButton.
			URL helpIconURL = param.getHelpImageURL();
			JButton helpButton = null;
			
			if ( helpIconURL != null ) {
				helpButton = new JButton( new ImageIcon(helpIconURL) );
				helpButton.setBorderPainted(false);
			} else
				helpButton = new JButton( "Help" );
			
			helpButton.setAlignmentX( JToggleButton.CENTER_ALIGNMENT );
			helpButton.setAlignmentY( JToggleButton.CENTER_ALIGNMENT );
			helpButton.setMargin( new Insets( 0, 0, 0, 0 ) );
			helpButton.addActionListener(new ButtonL());
			helpButton.setToolTipText("Displays information/help on this parameter.");
			bottomPanel.add( helpButton );
			
		}
	}

	//-----------------------------------------------------------------------------
	private boolean firstTime;
	
	/**
	*  Reads the value input in the text field and puts it into
	*  the corresponding parameter.  If the text edit field is empty,
	*  the parameter is made undefined.
	**/
	public void getInput() {
		double newValue = theTField.getValue();
		
		if (Double.isNaN(newValue))
			theParam.makeUndefined();

		else {
			//	If the value has changed, reset the parameter.
			float floatValue = (float)newValue;
			if ( floatValue != oldValue ) {
				theParam.setValue( floatValue );
				
				//	Verify that the parameter the user has input is valid.
				ParamVerifier verifier = ((AdvParam) theParam).getVerifier();
				if ( verifier != null ) {
					float prevValueRU = theParam.getValueRU();
					floatValue = prevValueRU;
					Component root = SwingUtilities.getRoot((Component)this);
					floatValue = verifier.check( root, floatValue );
					
					//	Call parameter verification method.
					if ( floatValue != prevValueRU ) {
						theParam.setValueRU( floatValue );
						
						//	Verification method changed value of param.
						resetText();
					}
				}
			}
		}
		
	}

	/**
	*  Defines a button listener for our help button.
	**/
	private class ButtonL extends Object implements ActionListener {

		public  void actionPerformed( ActionEvent evt ) {
			
			//	Extract the help information from the parameter.
			HelpSystem help = ((AdvParam) theParam).getHelpSystem();
			
			//	Show the help information for this parameter.
			if ( help != null )
				help.showHelp();
			
		}

	}
	
	
}


