/*
*   AutoParam - An automatically updating parameter object.
*
*   Copyright (C) 1998-2004 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Lesser General Public License for more details.
*
*   You should have received a copy of the GNU Lesser General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*   Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.tools.ParamTools;

import java.io.IOException;
import java.net.URL;
import java.util.Observable;
import java.util.Observer;
import java.util.ArrayList;
import java.util.Iterator;
import java.text.ParseException;

import jahuwaldt.tools.units.Unit;
import jahuwaldt.tools.units.CannotConvertException;


/**
*  <p>  An AdvParam parameter object that can automatically update
*       it's value based on other parameters in a given
*       parameter database (see class ParamDatabase).  </p>
*
*  <p>  To use this class, create AutoParam objects for
*       each parameter in your problem.  AutoParam objects will
*       automatically be added to the given ParamDatabase object.
*       Then call AutoParam.linkAllParams() to instruct the parameters
*       the the ParamDatabase to link up with each other.  After this,
*       parameters will automatically maintain the links between
*       themselves as values are passed to them (usually with
*       an AutoParamInput textbox).
*
*  <p>  Modified by:  Joseph A. Huwaldt   </p>
*
*  @author  Joseph A. Huwaldt   Date: January 13, 1998
*  @version May 23, 2002
*  @see ParamDatabase
**/
public class AutoParam extends AdvParam implements Observer {

	/**
	*  Debugging flag for this class.
	**/
	private static final boolean DEBUG = false;

	/**
	*  Reference to the parameter database.
	**/
	private ParamDatabase pdb;

	//	The observer object that notifies "children" objects.
	private transient Observable notifier = null;

	/**
	*  Determines if the parameter is manually input.
	**/
	private boolean manual = false;

	/**
	*  Ref. to the current method for calc this param.
	**/
	private ParamMethod currMethod = null;

	/**
	* List of method input parameter label strings.
	**/
	private String[] currInputs;

	/**
	*  Index into availMethods of current method.
	**/
	private int currMethodIdx = - 1;

	/**
	*  List of available methods for this parameter.
	**/
	protected transient ArrayList availMethods = new ArrayList();

	/**
	*  Array of input values used by updateValue() only.
	**/
	private transient float[] inputValues = null;

	//-----------------------------------------------------------------------------
	/**
	*  Constructs an AutoParam object that takes a parameter label,
	*  units, and a method definition.
	*
	*  @param  label      The label to be attached to this object.
	*  @param  theUnits   The units associated with this parameter.
	*  @param  theMethod  Handle to a ParamMethod object to calculate this
	*                     parameter with.
	*  @param  theDatabase  The parameter database this parameter should
	*                       use to lookup other parameters.
	*  @exception  IllegalArgumentException  Thrown if one of the input
	*              parameters is blank.
	**/
	public AutoParam( String label, Unit theUnits, ParamMethod theMethod,
								ParamDatabase theDatabase ) throws IllegalArgumentException {
		super( label, theUnits );

		if ( theMethod != null ) {
			checkInputs( theMethod );	//	Have the input parameter strings been entered properly?
			availMethods.add( theMethod );
		}
		pdb = theDatabase;
		theDatabase.put( label, this );
	}

	/**
	*  Constructs an AutoParam object that takes a parameter label
	*  and a method definition.
	*
	*  @param  label      The label to be attached to this object.
	*  @param  theMethod  Handle to a ParamMethod object to calculate this
	*                     parameter with.
	*  @param  theDatabase  The parameter database this parameter should
	*                       use to lookup other parameters.
	*  @exception  IllegalArgumentException  Thrown if one of the input
	*              parameters is blank.
	**/
	public AutoParam( String label, ParamMethod theMethod,
								ParamDatabase theDatabase ) throws IllegalArgumentException {
		super( label );

		if ( theMethod != null ) {
			checkInputs( theMethod );	//	Have the input parameter strings been entered properly?
			availMethods.add( theMethod );
		}
		pdb = theDatabase;
		theDatabase.put( label, this );
	}

	/**
	*  Constructor that takes a parameter label, symbol image path string, units,
	*  and a method definition.
	*
	*  @param  label       The label to be attached to this object.
	*  @param  symbolURL   The URL to an image file to use in place of the text
	*                      label string.
	*  @param  theUnits    The units associated with this parameter.
	*  @param  theMethod   Handle to a ParamMethod object to calculate
	*                      this parameter with.
	*  @param  theDatabase The parameter database this parameter should
	*                      use to lookup other parameters.
	*  @exception  IllegalArgumentException  Thrown if one of the input
	*              parameters is blank.
	**/
	public AutoParam( String label, URL symbolURL, Unit theUnits,
								ParamMethod theMethod, ParamDatabase theDatabase )
								throws IllegalArgumentException {
		super( label, symbolURL, theUnits );

		if ( theMethod != null ) {
			checkInputs( theMethod );	//	Have the input parameter strings been entered properly?
			availMethods.add( theMethod );
		}
		pdb = theDatabase;
		theDatabase.put( label, this );
	}

	/**
	*  Constructor that takes a parameter label, symbol image path string, and a
	*  method definition.
	*
	*  @param  label       The label to be attached to this object.
	*  @param  symbolHost  The URL of the host containing the symbol image.  This is
	*                      usually the "document base".
	*  @param  symbolURL   The URL to an image file to use in place of the text
	*                      label string.
	*  @param  theMethod   Handle to a ParamMethod object to calculate this
	*                      parameter with.
	*  @param  theDatabase The parameter database this parameter should
	*                      use to lookup other parameters.
	*  @exception  IllegalArgumentException  Thrown if one of the input
	*              parameters is blank.
	**/
	public AutoParam( String label, URL symbolURL, ParamMethod theMethod,
							ParamDatabase theDatabase ) throws IllegalArgumentException {
		super( label, symbolURL );

		if ( theMethod != null ) {
			checkInputs( theMethod );	//	Have the input parameter strings been entered properly?
			availMethods.add( theMethod );
		}
		pdb = theDatabase;
		theDatabase.put( label, this );
	}

	/**
	*  Constructor that takes a parameter label, symbol image path string,
	*  units, and a list of method definitions.
	*
	*  @param  label       The label to be attached to this object.
	*  @param  symbolURL   The URL to an image file to use in place of the text
	*                      label string.
	*  @param  theUnits    The units associated with this parameter.
	*  @param  methodLst   List of methods used to calculate this parameter.
	*  @param  theDatabase The parameter database this parameter should
	*                      use to lookup other parameters.
	*  @exception  IllegalArgumentException  Thrown if one of the input
	*              parameters is blank.
	**/
	public AutoParam( String label, URL symbolURL, Unit theUnits, ParamMethod[] methodLst,
							ParamDatabase theDatabase ) throws IllegalArgumentException {
		super( label, symbolURL, theUnits );

		if (methodLst != null) {
			int length = methodLst.length;
			for (int i=0; i < length; ++i) {
				ParamMethod theMethod = methodLst[i];
			
				if ( theMethod != null ) {
					//	Have the input parameter strings been entered properly?
					checkInputs( theMethod );
					availMethods.add( theMethod );
				}
			}
		}
		
		pdb = theDatabase;
		theDatabase.put( label, this );
	}

	/**
	*  Constructor that takes a collection of AutoParam inputs.
	*
	*  @param  inputs  A collection of inputs that can be optionally
	*                  supplied to create an AutoParam object.
	*  @exception  IllegalArgumentException  Thrown if one of the required input
	*              parameters is blank or improperly defined.
	**/
	public AutoParam( AutoParamInputs inputs ) throws IllegalArgumentException,
															CannotConvertException {
		super( inputs.label, inputs.symbolURL, inputs.theUnits );

		if (inputs.label == null || inputs.label.length() == 0)
			throw new IllegalArgumentException("AutoParam objects require a label.");
		
		if (inputs.methodLst != null) {
			int length = inputs.methodLst.length;
			for (int i=0; i < length; ++i) {
				ParamMethod theMethod = inputs.methodLst[i];
			
				if ( theMethod != null ) {
					//	Have the input parameter strings been entered properly?
					checkInputs( theMethod );
					availMethods.add( theMethod );
				}
			}
		} else if ( inputs.theMethod != null ) {
			//	Have the input parameter strings been entered properly?
			checkInputs( inputs.theMethod );
			availMethods.add( inputs.theMethod );
		}
		
		pdb = inputs.theDatabase;
		pdb.put( inputs.label, this );
		
		if (inputs.availUnits != null)
			setAvailUnits(inputs.availUnits);
		
		if (inputs.numFormat != null)
			setNumberFormat(inputs.numFormat);
		
		setParamVerifier(inputs.verifierMethod);
		
		setDescription(inputs.desc);
	}

	//-----------------------------------------------------------------------------
	/**
	*  Checks that the input method and strings of parameters exist.
	*
	*  @param  theMethod  Handle to a ParamMethod object to calculate this
	*                     parameter with.
	*  @exception  IllegalArgumentException  Thrown if inputArr contains a null string.
	**/
	private final void checkInputs( ParamMethod theMethod ) throws IllegalArgumentException {
		int length = theMethod.getNumInputs();
		
		for ( int i = 0; i < length; ++i ) {
			if ( theMethod.getInputLabel( i ) == null ) {
				throw new IllegalArgumentException( "Exception: AutoParam("
				+ this.getLabel() + ") ParamMethod(" + theMethod.getClass().getName()
				+ ") input label #" + i + " is null!" );
			}
		}
	}

	/**
	*  Have all parameters in a parameter database link up to each other
	*  (link parent parameters to children parameters).  This is usually called
	*  soon after all the parameter objects for a particular problem have
	*  been defined and added to the database.
	*
	*  @param  theDatabase  The parameter database where we want all the
	*                       parameters linked together.
	*  @throws ParamException with a descriptive message if a problem is found.
	**/
	public static void linkAllParams( ParamDatabase theDatabase ) throws ParamException {
		AutoParam theParam;
		
		//	Get enumeration of elements in database.
		Iterator dbElements = theDatabase.iterator();
		while ( dbElements.hasNext() ) {
		
			//	Get the next element from database.
		    Object next = dbElements.next();
		    if (next instanceof AutoParam) {
			    theParam = (AutoParam)next;
			
			    //	Check this parameter's methods for valid input parameters.
			    theParam.checkMethods();
			
			    //	Link this param to the params it depends on.
			    theParam.linkToParents();
		    }
		}
		
	}

	/**
	*  Clears all the values from all the parameters in a database.
	*  Makes ALL the parameters undefined.
	*
	*  @param  theDatabase  The parameter database from which we want
	*                       ALL the parameters cleared.
	**/
	public static void clearAllParams( ParamDatabase theDatabase ) {
		Param theParam;
		
		//	Get enumeration of elements in database.
		Iterator dbElements = theDatabase.iterator();
		
		while ( dbElements.hasNext() ) {
		
			//	Get the next element from database.
			theParam = (Param)dbElements.next();
			
			//	Make the parameter undefined.
			if ( theParam.isDefined() ) {
				theParam.makeUndefined();
				if (theParam instanceof AutoParam)
				    ((AutoParam)theParam).makeAutoAndUpdate();
			}
		}
	}

	/**
	*  Method to make this parameter undefined.  Overrides method in
	*  Param class.
	**/
	public void makeUndefined() {
		if ( isDefined() ) {
		
			super.makeUndefined();			//	Call original makeUndefined() method.
			updateChildren();				//	Inform children that this param is undefined.
			
			if ( currMethod != null )
				clearCurrMethod();			//	Clear the current method, if it exists.
			
		}
		
	}

	/**
	*  Returns true if parameter has been input manually, false if it
	*  is automatically calculated.
	*
	*  @return  Returns true if this parameter has been manually input.
	**/
	public final boolean isManuallyInput() {
		return (manual || isManualOnly());
	}

	/**
	*  Returns true if this parameter can only be input manually
	*  (it can not be automatically calculated).
	**/
	public final boolean isManualOnly() {
		return (currMethod == null && availMethods.size() == 0);
	}
	
	/**
	*  Method to tell this parameter that the value has been
	*  manually input.  Removes all links to parameters this
	*  one was dependent on.
	**/
	public void makeManuallyInput() {
		manual = true;
		
		//	Clear the current method (since we are now manual).
		clearCurrMethod();
		
	}

	/**
	*  Method to tell this parameter that the value should be
	*  automatically calculated.  This object will also link up with
	*  all the parameters it is dependent on (parents) when this
	*  method is called.  If there are no methods available to this
	*  parameter, then nothing will happen.
	**/
	public void makeAuto() {
	
		if (!isManualOnly() ) {
			manual = false;
		
			//	Create links to between this parameter and method input parameters.
			linkToParents();
		}
		
	}

	/**
	*  Method to tell this parameter that the value should be
	*  automatically calculated and that it should update it's
	*  value and notify it's observers.
	**/
	public void makeAutoAndUpdate() {
		makeAuto();
		updateValue();
		updateChildren();
	}
	
	/**
	*  Adds a new calculation method to the list of those available for use
	*  by this parameter.
	*
	*  @param  newMethod  The ParamMethod to be added to the list of available methods.
	*  @exception  IllegalArgumentException  Thrown if inputArr contains a null string.
	**/
	public void addMethod( ParamMethod newMethod ) throws IllegalArgumentException {
	
		if ( newMethod != null ) {
			//	Have the input parameter strings been entered properly?
			checkInputs( newMethod );
			availMethods.add( newMethod );
		}
	}

	/**
	*  Clears the current method and associated data structures.
	*  Breaks links to parents of the current method.
	**/
	protected void clearCurrMethod() {
	
		//	Break links to old method input parameters.
		removeParents();
		
		//	Clear the current method data structures.
		currMethod = null;
		currInputs = null;
		currMethodIdx = -1;
	}

	/**
	*  Determines if a method exists for automatically calculating this
	*  parameter.  If a valid method is found, the currMethod is updated.
	*
	*  @return  Returns true if inputs exist and are defined for calculating
	*           this parameter.
	**/
	protected boolean isAvailable() {
		int size, length;
		boolean goodMethod;
		Param theParam;
		ParamMethod aMethod;
		
		if ( pdb.isEmpty() ) {
			return false;
		}
		
		//	First see if our "current" method will work (if it worked before, it might now).
		if ( currMethod != null ) {
			goodMethod = true;
			length = currInputs.length;
			int i = 0;
			
			while ( i < length ) {
				theParam = pdb.get( currInputs[i] );
				if ( theParam == null || !theParam.isDefined() ) {
					goodMethod = false;
					break;
				}
				++i;
			}
			if ( goodMethod == true ) {
				//	Make sure the inputs are appropriate for this method.
				if (currMethod.validInputs(pdb, currInputs))
					return true;
			}
		}
		
		//	Loop over available methods until we find one that will work.
		size = availMethods.size();
		for ( int j = 0; j < size; ++j ) {

			//	Don't repeat the "current" method case.
			if ( j != currMethodIdx ) {
			
				//	Retrieve method input parameter information.
				String[] mInputs = ((ParamMethod) availMethods.get( j )).getAllLabels();
				
				//	Loop over all the inputs to this method.
				goodMethod = true;
				length = mInputs.length;
				int i = 0;
				
				while ( i < length ) {
					theParam = pdb.get( mInputs[i] );
					if ( theParam == null || ! theParam.isDefined() ) {
						goodMethod = false;
						break;
					}
					++i;
				}
				
				ParamMethod newMethod = (ParamMethod)availMethods.get(j);
				if ( goodMethod == true && newMethod.validInputs(pdb, mInputs)) {
					// A good working method was found!
					
					//	Clear the current method (that we are about to replace).
					clearCurrMethod();
					
					//	Change method and input parameters.
					currMethod = newMethod;
					currInputs = mInputs;
					currMethodIdx = j;
					
					//	Create links to new method input parameters.
					linkToParents();
					return true;
				}
			}
		}

		return false;
	}

	/**
	*  Checks to see if the necessary input parameters are defined.  If they are
	*  this parameter's value is updated with those input values.
	**/
	void updateValue() {
	
		if ( isManuallyInput() )	return;
		
		//	Check to see if we can update this parameter.
		if ( isAvailable() ) {
			Param theParam;
			float value;
			int length = currInputs.length;
			
			//	Allocate space for input values if needed.
			if ( inputValues == null || inputValues.length != length ) {
				inputValues = new float [ length ];
			}
			
			//	Extract input values from their parameter objects.
			for ( int i = 0; i < length; ++i ) {
			
				//	Get the correct parameter from the database.
				theParam = pdb.get( currInputs[i] );
				
				//	Get parameter's current value in reference units.
				value = theParam.getValueRU();
				
				//	Store the value in the input array.
				inputValues[i] = value;
			}
			
			try {
				//	Calculate the value (in reference units) of this parameter using it's calc method.
				value = currMethod.calc( inputValues );
				
			} catch (Exception e) {
				// An error occured while trying to calculate this parameter.
				// Inform the object calling this one of the problem.
				makeUndefined();
				makeAuto();
				throw new IllegalArgumentException(getLabel());
			}

			float old_value = this.getValueRU();
			if ( old_value != value ) {
				
				//	Save off the new value of this parameter and update children.
				this.setValueRU( value );
					
				//	Tell objects dependent on this one that they need to update themselves.
				updateChildren();
			}
			
		} else {
			//	If parameter isn't available, but it is auto-calculating, make it undefined.
			if ( isDefined() ) {
				makeUndefined();
				makeAuto();
			}
			
		}
		
	}

	/**
	*  Add an object to the list of objects that are dependent on
	*  this parameter.  When this parameter changes value, all
	*  the observers are notified.
	*
	*  @param  observer  The object that is dependent on
	*                    (is a child of -- observes) this parameter.
	**/
	public void addObserver( Observer observer ) {
		if ( notifier == null )
			notifier = new ParamObservable();
		
		if ( DEBUG )
			System.out.println( "\t" + observer + " is child of " + getLabel() );
		
		notifier.addObserver( observer );
	}

	/**
	*  Removes an object from the list of objects that are dependent on
	*  this parameter.  The specified object will no longer be notified
	*  when this parameter changes value.
	*
	*  @param  observer  The object that is no longer dependent on this parameter.
	**/
	public void deleteObserver( Observer observer ) {
	
		if ( DEBUG )
			System.out.println( "\t" + observer + " no longer a child of " + getLabel() );
		
		if ( notifier != null )
			notifier.deleteObserver( observer );
	}

	/**
	*  Tell each object that this one depends on (each parent) that this
	*  object needs updating when they change value.  This is normally
	*  called when the calculation method used by the parameter has changed.
	**/
	protected void linkToParents() {
		AutoParam theParam;
		
		if ( DEBUG )
			System.out.println( "\nCreating links between " + this.getLabel() + " and:" );
		
		if ( currMethod != null ) {
			//	Link to the current method's input parameters.
			//	Add this object as a child to (dependent on) all the input parameters.
			int length = currInputs.length;
			
			for ( int i = 0; i < length; ++i ) {
				theParam = (AutoParam) pdb.get( currInputs[i] );
				theParam.addObserver( this );
			}
			
		} else {
			//	Link to all available method's input parameters.
			int j;
			int numUsedParams = 0;	//	Num params used as parents of this param.
			int numParamsTot = 0;	//	Total num of parent params in all the available input methods.
			String[] mInputs;
			AutoParam[] inputPArr;	//	Array of pointers to parents that have been linked to.
			
			//	Count up the number of parameters in all the input methods.
			int numMethods = availMethods.size();
			for ( j = 0; j < numMethods; ++j ) {
				mInputs = ((ParamMethod) availMethods.get( j )).getAllLabels();
				numParamsTot += mInputs.length;
			}
			
			//	Allocate memory for used parameter (parent) array.
			inputPArr = new AutoParam [ numParamsTot ];
			for ( j = 0; j < numMethods; ++j ) {

				//	Retrieve method input parameter information.
				mInputs = ((ParamMethod) availMethods.get( j )).getAllLabels();
				
				//	Add this object as a child to (dependent on) all this method's input parameters
				int length = mInputs.length;
				
				for ( int i = 0; i < length; ++i ) {
					theParam = (AutoParam) pdb.get( mInputs[i] );
					
					//	Make sure that we don't link to the same parent twice.
					//	Search through all the parents already linked to.
					boolean found = false;
					int pos = 0;
					while ( pos < numUsedParams && ! found ) {
						if ( theParam == inputPArr[pos] )
							found = true;
						else
							++pos;
					}
					
					//	If this parent wasn't in the list, then link to it and add it to the list.
					if ( found == false ) {
						theParam.addObserver( this );
						inputPArr[numUsedParams] = theParam;
						++numUsedParams;
					}
				}
			}
		}
	}

	/**
	*  Method that checks the validity of all the methods used by this
	*  parameter.  Throws an exception with a descriptive message if a problem is found.
	**/
	private void checkMethods() throws ParamException {
		
		//	Loop over all the methods used by this parameter.
		int numMethods = availMethods.size();
		for (int j = 0; j < numMethods; ++j ) {
			String[] mInputs = ((ParamMethod) availMethods.get( j )).getAllLabels();
			
			//	Loop over all the inputs to this method.
			int numInputs = 0;
			if (mInputs != null)	numInputs = mInputs.length;
			for (int i=0; i < numInputs; ++i) {
				String methodPLabel = mInputs[i];
				Param methodParam = (Param)pdb.get(methodPLabel);
				if (methodParam == null) {
					String methodName = ((ParamMethod)availMethods.get(j)).getClass().getName();
					throw new ParamException("Input \"" + methodPLabel + "\" of method \"" +
								methodName + "\" of param \"" + this.getLabel() + "\" not found.");
				}
			}
		}

	}
	
	/**
	*  Break the link between this object and all the objects that this
	*  one depends on (parent objects).  This is normally called before
	*  you change the method being used to calculate this parameter.  That
	*  way the old parent parameters won't keep sending update messages to
	*  this paramter.
	**/
	protected void removeParents() {
		AutoParam theParam;
		
		if ( DEBUG )
			System.out.println( "\nRemoving links between " + this.getLabel() + " and:" );
		
		if ( currMethod != null ) {
			//	Break links to current method input parameters.
			//	Remove this object from the child lists of all the input parameters.
			int length = currInputs.length;
			
			for ( int i = 0; i < length; ++i ) {
				theParam = (AutoParam) pdb.get( currInputs[i] );
				theParam.deleteObserver( this );
			}
			
		} else {
			//	Break links to all available method input parameters.
			int j;
			int numUsedParams = 0;	//	Num params used as parents of this param.
			int numParamsTot = 0;	//	Total num of parent params in all the available input methods.
			String[] mInputs;
			AutoParam[] inputPArr;	//	Array of pointers to parents that have been "unlinked".
			
			//	Count up the number of parameters in all the input methods.
			int numMethods = availMethods.size();
			for ( j = 0; j < numMethods; ++j ) {
				mInputs = ((ParamMethod) availMethods.get( j )).getAllLabels();
				numParamsTot += mInputs.length;
			}
			
			//	Allocate memory for already unlinked parameter (parent) array.
			inputPArr = new AutoParam [ numParamsTot ];
			for ( j = 0; j < numMethods; ++j ) {
			
				//	Retrieve method input parameter information.
				mInputs = ((ParamMethod) availMethods.get( j )).getAllLabels();
				
				//	Remove this object from the child lists of all the input parameters.
				int length = mInputs.length;
				
				for ( int i = 0; i < length; ++i ) {
					theParam = (AutoParam) pdb.get( mInputs[i] );
					
					//	Make sure that we don't unlink the same parent twice.
					//	Search through all the parents already unlinked.
					boolean found = false;
					int pos = 0;
					while ( pos < numUsedParams && ! found ) {
						if ( theParam == inputPArr[pos] )
							found = true;
						else
							++pos;
					}
					
					//	If this parent wasn't in the list, then unlink it and add it to the list.
					if ( found == false ) {
						theParam.deleteObserver( this );
						inputPArr[numUsedParams] = theParam;
						++numUsedParams;
					}
				}
			}
		}
	}

	/**
	*  Tell each object that is dependent on this one to update themselves.
	*  This is generally called automatically.
	**/
	public void updateChildren() throws IllegalArgumentException {
		if ( DEBUG )
			System.out.println( "Update Children of " + this.getLabel() + ":" );
		
		if ( notifier != null )
			notifier.notifyObservers( this );
	}

	/**
	*  Called automatically by the Observable notification system when one
	*  of the parameters this object is dependent on has changed.
	*
	*  @param  o    The object being observed (this object)
	*  @param  arg  The object that this one depends on that has changed.
	**/
	public void update( Observable o, Object arg ) {
		AutoParam parentParam = (AutoParam) arg;
		
		if ( DEBUG ) 
			System.out.println( "*\tUpdating " + this.getLabel() + " from " + parentParam.getLabel() );
		
		if ( !parentParam.isDefined() && this.isDefined() ) {
			//	The parent has become undefined!
			makeUndefined();
			makeAuto();
		}
		
		updateValue();
			
	}

	/**
	*  <p>  This object allows us to implement the observable interface
	*       in our AutoParam objects.
	*  </p>
	*
	*  <p>  Modified by:  Joseph A. Huwaldt  </p>
	*
	*  @author   Joseph A. Huwaldt   Date: February 2, 1998
	*  @version  June 25, 2000
	**/
	private class ParamObservable extends Observable {

		public void notifyObservers( Object b ) {
			setChanged();
			super.notifyObservers( b );
		}

	}


	//-----------------------------------------------------------------------------
	/**
	*  During serialization, this will handle the serialization of
	*  the list of methods associated with this parameter.
	**/
	private void writeObject( java.io.ObjectOutputStream out ) throws IOException {
	
		// Call the default write object method.
		out.defaultWriteObject();

		// Write out the number of methods we are going to write out.
		int size = availMethods.size();
		out.writeInt( size );

		// Now loop over all the methods and write out each one.
		for (int i=0; i < size; ++i) {
			ParamMethod method = (ParamMethod) availMethods.get(i);
			out.writeObject( method );
		}

	}

	/**
	*  During serialization, this will handle the reading in of
	*  the list of methods associated with this parameter.
	**/
	private void readObject( java.io.ObjectInputStream in ) throws IOException, ClassNotFoundException {
	
		// Call the default read object method.
		in.defaultReadObject();

		// Re-create the available memthods list object.
		availMethods = new ArrayList();
		
		// Read in the number of methods in the file.
		int size = in.readInt();

		// Loop over each method and read it in.
		for ( int i = 0; i < size; ++i ) {
			ParamMethod method = (ParamMethod) in.readObject();
			
			// Add the new method to the method list.
			availMethods.add( method );
		}

	}

}

