/*
*   AutoParamInput - Object used for entering AutoParam values using a text field.
*
*   Copyright (C) 1998-2004 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Lesser General Public License for more details.
*
*   You should have received a copy of the GNU Lesser General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*   Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.tools.ParamTools;

import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;
import java.text.ParseException;
import java.util.Observer;
import java.util.Observable;
import java.net.MalformedURLException;
import javax.swing.*;
import jahuwaldt.tools.ParamTools.*;


/**
*  Defines a panel object that is used for entering
*  parameter values (using a text field).  This differs
*  from "ParamInput" objects in that these objects
*  will automatically call update methods for AutoParam
*  parameters.
*
*  <p>  Modified by:  Joseph A. Huwaldt   </p>
*
*  @author  Joseph A. Huwaldt   Date: January 13, 1998
*  @version November 5, 2004
**/
public class AutoParamInput extends AdvParamInput implements Observer {

	private JCheckBox manInputCB;
	
	/**
	*  Constructs an AutoParamInput object using the given parameter.
	*
	*  @param  The parameter object that this input field should use.
	*  @throws MalformedURLException if either the path to the symbol or help icon image
	*          files could not be combined with the application location and turned into a URL.
	**/
	public AutoParamInput( AutoParam param ) throws MalformedURLException {
		super( (AdvParam) param );

		// Add a new focus and action listener to this ParamInput's text field.
		theTField.removeFocusListener(tfFocusListener);
		theTField.removeActionListener(tfActionListener);
		
		tfFocusListener = new TextFieldFocusL2();
		tfActionListener = new TextFieldListener2();
		
		theTField.addFocusListener( tfFocusListener );
		theTField.addActionListener( tfActionListener );

		//	Add a checkbox item that indicates if the object is manually input.
		manInputCB = new JCheckBox( "Manually Input", param.isManuallyInput() );
		manInputCB.addActionListener(new CheckboxListener());
		manInputCB.setFont( new Font( "SansSerif", Font.PLAIN, 9 ) );
		manInputCB.setToolTipText("Parameter is not auto-calculated if checked.");
		bottomPanel.add( manInputCB );
		
		// If the parameter can only be manually input, disable the manual input checkbox.
		if (param.isManualOnly())
			manInputCB.setEnabled(false);
		
		//	Make this text parameter input object an observer (child) of the parameter.
		param.addObserver(this);
		
		//	Record current value of this parameter.
		this.setOldValue();
	}

	//-----------------------------------------------------------------------------
    /**
    *  Sets the specified boolean to indicate whether or not this
    *  parameter input field should be editable.
    **/
    public void setEditable(boolean b) {
		super.setEditable(b);
        manInputCB.setEnabled(b);
    }
    
	/**
	*  Reads the value input in the text field and puts it into
	*  the corresponding parameter.  If the TextEdit field is empty,
	*  the parameter is made undefined.
	**/
	public void getInput() {
		
		//	Was a blank entered?
		if ( theParam.isDefined() ) {
			String s = theTField.getText();
			
			if ( s == null || s.equals( "" ) ) {
				//	Make the parameter undefined, and manually input.
				((AutoParam) theParam).makeManuallyInput();
				theParam.makeUndefined();
				return;
			}
		}
		
		super.getInput();
	}

	/**
	*  Called automatically by the Observable notification system when
	*  the parameter this object is dependent on has changed.
	*
	*  @param  o    The object being observed (this object)
	*  @param  arg  The parameter that this one depends on that has changed.
	**/
	public void update( Observable o, Object arg ) {
		AutoParam parentParam = (AutoParam) arg;
		
		// Change the text display.
		resetText();
		
		// Make sure the manually input check box is up to date.
		manInputCB.setSelected( parentParam.isManuallyInput() );
		
	}

	/**
	*  Defines a listener for our check box.
	**/
	private class CheckboxListener extends Object implements ActionListener {

		public  void actionPerformed( ActionEvent evt ) {
			JCheckBox theCB = (JCheckBox)evt.getSource();
			AutoParam theAutoParam = (AutoParam) theParam;
			
			if ( !theCB.isSelected() ) {
				try {
				
					//	The user want's this param to be auto calculated.
					theAutoParam.makeUndefined();
					theAutoParam.makeAutoAndUpdate();
					
				} catch (Exception e) {
					// Shouldn't happen.
					e.printStackTrace();
				}
			} else {
				//	The user want's this to be manually input.
				theAutoParam.makeManuallyInput();
			}
		}
	}
	
	
	/**
	*  Handle an event that indicates that the user may have changed
	*  the value in the text field of this parameter manually.
	**/
	private void handleInput() {
		float prevValue=0, newValue=0;
			
		try {
			// Round off the old value to the current display format.
			String text = this.nf.format(theParam.getValue());
			prevValue = this.nf.parse(text).floatValue();
			
			this.getInput();
			
			// Round off the new value to the current display format.
			text = this.nf.format(theParam.getValue());
			newValue = this.nf.parse(text).floatValue();

		} catch (ParseException err) {
			// Shouldn't be possible.
			err.printStackTrace();
		}
			
		//	Has the value been changed by the user (manually input)?
		if ( ! Float.isNaN( newValue ) && newValue != prevValue ) {
			try {
				//	A new value was input.
				AutoParam theAutoParam = (AutoParam)theParam;
				oldValue = newValue;
				
				//	Tell this parameter that it has been manually input.
				theAutoParam.makeManuallyInput();
				
				//	Notify parameters that are dependent on this one that they need to update.
				theAutoParam.updateChildren();
				
			} catch (Exception err) {
				JOptionPane.showMessageDialog ( this, "The value entered caused an error in calculating "
								+ err.getMessage() + ".",
								"Update Error",
								JOptionPane.ERROR_MESSAGE );
				((AutoParam)theParam).makeUndefined();
				((AutoParam)theParam).makeAuto();
			}
		}
	}
	
	private boolean keyPressed=false;
	/**
	*  Defines a listener for our text field that is called whenever the user
	*  types in a new number and presses enter.
	**/
	private class TextFieldListener2 extends Object implements ActionListener {

		public void actionPerformed( ActionEvent e ) {
			keyPressed = true;
			AutoParamInput.this.handleInput();
			keyPressed = false;
		}
	}
	
	
	/**
	*  Defines a listener for our text field that is called whenever the
	*  focus changes off a particular text field.
	**/
	private class TextFieldFocusL2 extends Object implements FocusListener {

		public void focusGained( FocusEvent e ) { }

		public void focusLost( FocusEvent e ) {
			if (!keyPressed)
				AutoParamInput.this.handleInput();
			keyPressed = false;
		}

	}

}


