/*
*   Param - Base class for all parameter value objects.
*
*   Copyright (C) 1998-2004 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Lesser General Public License for more details.
*
*   You should have received a copy of the GNU Lesser General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*   Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.tools.ParamTools;

import java.text.NumberFormat;
import java.text.ParseException;

import jahuwaldt.tools.units.Unit;
import jahuwaldt.tools.units.CannotConvertException;


/**
*  A generic parameter class for use in my engineering
*  programs.  The Param object provides a package that
*  contains an extended amount of information on a single
*  parameter or value such as the units and an identifying
*  label.
*
*  <p>  Modified by:  Joseph A. Huwaldt    </p>
*
*  @author  Joseph A. Huwaldt   Date: November 23, 1997
*  @version November 6, 2004
**/
public class Param implements java.io.Serializable {

	/**
	*  The value of this parameter.
	**/
	private float theValue;

	/**
	* Is this parameter defined?
	**/
	private boolean defined;

	/**
	*  String containing a label for this parameter.
	**/
	private String labelStr;

	/**
	*  The units of this parameter object.
	**/
	private Unit theUnits;

	/**
	*  Value of this parameter in the reference units.
	**/
	private float valueRU;

	/**
	*   Array of available units for this parameter.
	**/
	protected Unit[] availUnits;

	/**
	*  The number format used when displaying this parameter.
	**/
	private NumberFormat numFormat = null;
	
	
	//-----------------------------------------------------------------------------
	/**
	*  Constructs a parameter object using a label string.
	*  Sets the parameter to an initial value of Float.NaN
	*  (makes parameter undefined).
	*
	*  @param  The label to be attached to this object.
	**/
	public Param( String label ) {
		labelStr = label;
		theValue = Float.NaN;
		valueRU = Float.NaN;
		defined = false;
	}

	/**
	*  Constructs a parameter object using a label string
	*  and a float value for the parameter.
	*
	*  @param   label  The label to be attached to this object.
	*  @param   value  The initial value for the parameter to hold.
	**/
	public Param( String label, float value ) {
		labelStr = label;
		theValue = value;
		valueRU = value;
		defined = true;
	}

	/**
	*  Constructs a parameter object using a label string
	*  and the units for the parameter while setting the
	*  parameter's initial value to Float.NaN (undefined).
	*
	*  @param  label    The label to be attached to this object.
	*  @param  theUnits The units associated with this parameter.
	**/
	public Param( String label, Unit theUnits ) {
		labelStr = label;
		this.theUnits = theUnits;
		theValue = Float.NaN;
		valueRU = Float.NaN;
		defined = false;
	}

	/**
	*  Constructs a parameter object using a label string,
	*  float value, and units for the parameter.
	*
	*  @param  label    The label to be attached to this object.
	*  @param  value    The initial value for the parameter to hold.
	*  @param  theUnits The units associated with this parameter.
	**/
	public Param( String label, float value, Unit theUnits ) {
		labelStr = label;
		this.theUnits = theUnits;
		defined = true;
		theValue = value;
		
		//	Store "value" in the reference units for this type of unit.
		if ( theUnits != null )
			valueRU = (float) theUnits.convertToRef( value );
		else
			valueRU = value;
		
	}

	//-----------------------------------------------------------------------------
	/**
	*  Method to return the current value of this parameter in the
	*  current units.
	*
	*  @return  The float value of this parameter in the current units is returned.
	**/
	public final float getValue() {
		return theValue;
	}

	/**
	*  Method to return the current value of this parameter in the parameter's
	*  reference units (if the parameter has been assigned a unit type).
	*
	*  @return  The float value of this parameter in reference units is returned.
	**/
	public final float getValueRU() {
		return valueRU;
	}

	/**
	*  Method to set the value of this parameter in the current units.
	*  Makes the parameter "defined".
	*
	*  @param   value  The value that the parameter object should be set to.
	**/
	public void setValue( float value ) {
		theValue = value;
		
		if ( !Float.isNaN( value ) ) {
		
			//	Convert this value to the reference units for this parameter's unit type.
			if ( theUnits != null )
				valueRU = (float) theUnits.convertToRef( value );
			else
				valueRU = value;
			
			defined = true;
			
		} else
			makeUndefined();
		
	}

	/**
	*  Method to set the value of this parameter in the current units
	*  using string input.  Makes this parameter "defined".
	*
	*  @param      value   The value that the parameter object should be set to.
	*  @exception  NumberFormatException, ParseException   Thrown if
	*              value string can't be converted to a float.
	**/
	public void setValue( String value ) throws NumberFormatException, ParseException {
		if ( value == null || value.equals( "" ) )
			makeUndefined();
			
		else {
			if (numFormat == null)
				theValue = Float.valueOf( value ).floatValue();
			else
				theValue = numFormat.parse(value).floatValue();
			
			if ( !Float.isNaN( theValue ) ) {
			
				//	Convert this value to the reference units for this parameter's unit type.
				if ( theUnits != null )
					valueRU = (float) theUnits.convertToRef( theValue );
				else
					valueRU = theValue;
				
				defined = true;
				
			} else
				makeUndefined();
			
		}
	}

	/**
	*  Method to set the value of this parameter in reference units.  Makes
	*  the parameter "defined".  Does not change the current units of the
	*  parameter.
	*
	*  @param   value   The value in reference units that the parameter object
	*                   should be set to.
	**/
	public void setValueRU( float value ) {
		valueRU = value;
		
		if ( !Float.isNaN( value ) ) {
		
			//	Convert this value to the current units from the reference units.
			if ( theUnits != null )
				theValue = (float) theUnits.convertFromRef( value );
			else
				theValue = value;
			
			defined = true;
			
		} else
			makeUndefined();
		
	}

	/**
	*  Method to return the string representation of the label of this
	*  parameter.
	*
	*  @return  Returns the string representation of the parameter label.
	**/
	public final String getLabel() {
		return labelStr;
	}

	/**
	*  Method to change the label that is associated with this parameter.
	*
	*  @param   label   The label to be attached to this object.
	**/
	public void setLabel( String label ) {
		labelStr = label;
	}

	/**
	*  Method to return the current units of this parameter.
	*
	*  @return  Returns the units of this parameter.
	**/
	public final Unit getUnits() {
		return theUnits;
	}

	/**
	*  Method to change the units of this parameter without converting
	*  the value to the new units (value is left unchanged).
	*
	*  @param      newUnits   The new units to be assigned to this parameter.
	*  @exception  CannotConvertException   Unable to convert between the
	*              specified units.
	**/
	public void setUnits( Unit newUnits ) throws CannotConvertException {
		if ( newUnits == null )
			theUnits = null;
			
		else if ( theUnits == null )
			theUnits = newUnits;
			
		else {
			//	Make sure the new units are compatible with the old units.
			if ( ! Unit.compatible( newUnits, theUnits ) )
				throw new CannotConvertException( "newUnits are inconsistant with current units!" );
			theUnits = newUnits;
		}
	}

	/**
	*  Method to convert the value of this parameter from the current
	*  units to the input units (value is changed).
	*
	*  @param      newUnits   The new units to convert this parameter to.
	*  @exception  CannotConvertException   Unable to convert between the
	*              specified units.
	**/
	public void changeUnits( Unit newUnits ) throws CannotConvertException {
		if ( newUnits == null )
			theUnits = null;
			
		else if ( theUnits == null )
			theUnits = newUnits;
				
		else {
			//	Convert the value and change the units.
			theValue = (float) theUnits.convert( theValue, newUnits );
			theUnits = newUnits;
		}
	}

	/**
	*  Sets the list of unit types that are available for use
	*  by this parameter.  If this parameter does not already have units assigned,
	*  then the 1st unit of the unit list will become the current unit.
	*
	*  @param  newUnitArr  The array of Units to be made into the available unit list.
	*  @exception  CannotConvertException  Unable to convert between the specified
	*              units.
	*  @exception  IllegalArgumentException  Thrown if newUnitArr contains a null
	*              element.
	**/
	public void setAvailUnits( Unit[] newUnitArr ) throws CannotConvertException,
													IllegalArgumentException {
		int i;
		
		// First check to see if the user is zeroing out the list of units.
		if ( newUnitArr == null ) {
			availUnits = null;
			return;
		}
		
		// Make sure none of the units are blank.
		for ( i = 0; i < newUnitArr.length; ++i ) {
			if ( newUnitArr[i] == null ) {
				throw new IllegalArgumentException( "Param(" + getLabel() + ") setAvailUnits newUnitArr["
								+ i + "] is null!" );
			}
		}
		
		if ( availUnits != null && availUnits.length > 0 ) {
		
			//	Test to see if a compatible unit type was passed to us.
			Unit testUnit = availUnits[0];
			for ( i = 1; i < newUnitArr.length; ++i ) {
				if ( ! Unit.compatible( testUnit, newUnitArr[i] ) ) {
					throw new CannotConvertException( "Param(" + getLabel() + ") setAvailUnits newUnitArr["
								+ i + "] not compatible with existing units!" );
				}
			}
			
			//	If we have gotten this far without an exception, then the types are compatible.
			//	Replace the old unit list with the new one.
			availUnits = newUnitArr;
			
		} else {
		
			//	Test to see if all the input units are compatible with each other.
			Unit testUnit = newUnitArr[0];
			for ( i = 1; i < newUnitArr.length; ++i ) {
				if ( ! Unit.compatible( testUnit, newUnitArr[i] ) ) {
					throw new CannotConvertException( "Param(" + getLabel() + ") setAvailUnits newUnitArr["
								+ i + "] inconsistant with other newUnitArr values!" );
				}
			}
			
			//	Are the new units compatible with the current parameter units?
			if ( theUnits != null ) {
				if ( ! Unit.compatible( testUnit, theUnits ) ) {
					throw new CannotConvertException( "Param(" + getLabel()
					+ ") setAvailUnits newUnitArr units are inconsistant with"
					+ " current parameter units!" );
				}
			}
			
			//	If we have gotten this far without an exception, then the types are compatible.
			availUnits = newUnitArr;
		}
		
		// If no current units are set, set them to the 1st unit in the list.
		if ( theUnits == null && availUnits != null && availUnits.length > 0 )
			setUnits(availUnits[0]);
		
	}

	/**
	*  Gets the list of unit types that are available for use
	*  by this parameter.  If this parameter does not have a list of
	*  available units, null is returned.
	*
	*  @return An array of unit types available with this parameter.
	**/
	public Unit[] getAvailUnits() {
		return availUnits;
	}

	/**
	*  Set the number format to use when displaying this parameter.
	*  If no number format is set, then a default will be used.
	**/
	public void setNumberFormat(NumberFormat nf) {
		numFormat = nf;
	}
	
	/**
	*  Get the number format used when displaying this parameter.
	*  If no number format has been set, null is returned.
	**/
	public NumberFormat getNumberFormat() {
		return numFormat;
	}
	
	/**
	*  Returns true if parameter is defined, false otherwise.
	*
	*  @return  Returns true if this parameter is defined.
	**/
	public final boolean isDefined() {
		return defined;
	}

	/**
	*  Method to make this parameter undefined.  The value is set to
	*  Float.NaN.
	*
	*  @return  Returns true if this parameter is defined.
	**/
	public void makeUndefined() {
		defined = false;
		theValue = Float.NaN;
		valueRU = Float.NaN;
	}

	/**
	*  Method to return the value of this parameter in current
	*  units as a string.
	*
	*  @return Returns a string representation of the value of this parameter
	*          in the current units.
	**/
	public String toString() {
		String s = null;
		if (numFormat == null)
			s = Float.toString( theValue );
		else
			s = numFormat.format(theValue);
			
		return s;
	}

}


