/*
*   ParamInput - Panel used for entering Param values using a text field.
*
*   Copyright (C) 1998-2004 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Lesser General Public License for more details.
*
*   You should have received a copy of the GNU Lesser General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*   Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.tools.ParamTools;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;
import java.text.NumberFormat;
import java.text.DecimalFormat;
import java.text.ParseException;

import jahuwaldt.tools.units.Unit;
import jahuwaldt.tools.units.CannotConvertException;
import jahuwaldt.swing.DecimalField;


/**
*  Defines an Swing JPanel object that is used for entering
*  Param type parameter values (using a text field).
*  ParamInput provides a user interface for interacting
*  with parameter objects.
*
*  <p>  Modified by:  Joseph A. Huwaldt    </p>
*
*  @author  Joseph A. Huwaldt   Date: November 23, 1997
*  @version November 6, 2004
**/
public class ParamInput extends JPanel {

	/**
	*  References to event listeners used by this panel's text field.
	**/
	protected FocusListener tfFocusListener;
	protected ActionListener tfActionListener;
	
	/**
	*  Value param had before becoming input focus (in ref units).
	**/
	protected float oldValue;

	/**
	*  TextField for inputting a value to this parameter.
	**/
	protected DecimalField theTField;
	
	/**
	*  The combo-box displaying the current units (when there are more than one).
	*  This variable has a value of null if there are no units or only 1 unit.
	**/
	private JComboBox unitList = null;
	
	/**
	*  The label used to display the units of this parameter when there is
	*  only one unit.  If this variable is null, then there are either no
	*  units at all or a list of units (more than one).
	**/
	private JLabel unitLabel = null;
	
	/**
	*  The number format object used to format numbers
	**/
	protected NumberFormat nf;

	/**
	* The parameter that this input panel refers to.
	**/
	protected Param theParam;

	//-----------------------------------------------------------------------------
	/**
	*  Don't allow the user to access the default constructor.
	**/
	protected ParamInput() { }

	/**
	*  Constructs a ParamInput class that accepts a Param object for input.
	*
	*  @param  The Param object that this input field should be associated with.
	**/
	public ParamInput( Param param ) {

		theParam = param;

		//	Create layout
		setLayout( new BorderLayout() );
			
		//	Create text input field for this parameter.
		nf = param.getNumberFormat();
		if (nf == null)
			nf = (DecimalFormat)NumberFormat.getInstance();
		
		if ( param.isDefined() )
			theTField = new DecimalField( param.getValue(), 10, nf );
		else
			theTField = new DecimalField( 10, nf );
		
		tfActionListener = new TextFieldListener();
		tfFocusListener = new TextFieldFocusL();
		theTField.addActionListener( tfActionListener );
		theTField.addFocusListener( tfFocusListener );
		add( "Center", theTField );
		
		//	Create the label for this parameter.
		JLabel theLabel = new JLabel( param.getLabel() + " =", JLabel.RIGHT );
		theLabel.setFont(new Font("Serif", Font.PLAIN, 12));
		theLabel.setLabelFor(theTField);
		add( "West", theLabel );
			
		//	Create the units label or choice menu for this parameter.
		if ( param.availUnits != null && param.availUnits.length > 1 ) {
		
			//	Many units are available, create a menu of them.
			unitList = new JComboBox(param.getAvailUnits());
			unitList.addActionListener(new UnitMenuListener());
			unitList.setToolTipText("Used to change parameter units.");
			
			//	Select the current units.
			unitList.setSelectedItem( param.getUnits() );
			add( "East", unitList );
			
		} else {
			//	Only 1 unit available, display as static string.
			String unitStr = " ";
			Unit theUnits = param.getUnits();
			if ( theUnits != null ) {
				unitStr += theUnits.toString();
			}
			unitStr += " ";
			
			unitLabel = new JLabel( unitStr, JLabel.LEFT );
			add( "East", unitLabel );
		}
		
	}

	//-----------------------------------------------------------------------------
	/**
	*  We are adding an inset to our ParamInput panel.  This is
	*  called by the container to determine our insets.
	*
	*  @return  An Insets object is returned.
	**/
	public Insets getInsets() {
		return new Insets( 0, 2, 0, 2 );
	}

	/**
	*  Reads the value input in the text field and puts it into
	*  the corresponding parameter.  If the TextEdit field is empty,
	*  the parameter is made undefined.
	**/
	public void getInput() {
		double newValue = theTField.getValue();
		
		if (Double.isNaN(newValue))
			theParam.makeUndefined();
		
		else {
			//	If the value has changed, reset the parameter.
			if ( (float)newValue != oldValue )
				//	Value changed by user, so reset it.
				theParam.setValue( (float)newValue );
		}
		
	}

	/**
	*  Method to return the current value of the parameter associated with
	*  this ParamInput object.  The value is returned in the
	*  current units.
	*
	*  @return  The float value of the parameter associated with this
	*           ParamInput object in the current units is returned.
	**/
	public final float getValue() {
		//	First retrieve any user inputs.
		this.getInput();
		
		//	Now return the parameter value.
		return theParam.getValue();
	}

	/**
	*  Method to return the current value of the parameter associated with
	*  this ParamInput object.  The value is returned in the reference units
	*  (if the parameter has been assigned a unit type).
	*
	*  @return  The float value of the parameter associated with this ParamInput
	*           object in reference units is returned.
	**/
	public final float getValueRU() {
		//	First retrieve any user inputs.
		this.getInput();
		return theParam.getValueRU();
	}

	/**
	*  Resets the text displayed in the edit text field to the
	*  current value of the parameter attached to this object.
	*  This should be called if the value of the parameter has changed
	*  without being input by the user (e.g.: by an internal calculation).
	**/
	public void resetText() {
	
		if ( theParam.isDefined() )
			theTField.setValue( theParam.getValue() );
		else
			theTField.setText( "" );
		
		//	If there are a list of units, make sure the combo box is
		//	displaying the correct one.
		if (unitList != null)
			unitList.setSelectedItem( theParam.getUnits() );
			
		//	If there is only a single unit, make sure it is correct.
		else if (unitLabel != null)
			unitLabel.setText(theParam.getUnits().toString());	
			
		setOldValue();
	}

    /**
    *  Sets the specified boolean to indicate whether or not this
    *  ParamInput should be editable.
    **/
    public void setEditable(boolean b) {
        theTField.setEditable(b);
    }
    
    /**
    *  Returns the boolean indicating whether this ParamInput
    *  is editable or not.
    **/
    public boolean isEditable() {
        return theTField.isEditable();
    }
    
	/**
	*  Set the old value of the parameter associated with this
	*  ParamInput object.  This is used internally to help
	*  determine if a new parameter value has been manually input
	*  by the user, or set in some other way (e.g.: by calculation).
	*  The old value is in the current units.
	**/
	protected void setOldValue() {
		try  {
		
			if ( theParam.isDefined() ) {
				oldValue = theParam.getValue();
				
				// Round off the value to the current display format.
				String text = nf.format(oldValue);
				oldValue = nf.parse(text).floatValue();
				
			} else
				oldValue = Float.NaN;
			
		} catch( ParseException err ) {
			// Shouldn't be possible.
			err.printStackTrace();
		}
		
	}
	
	/**
	*  Method to return the value of this parameter as a string.
	*
	*  @return Returns a string representation of the value of this
	*          parameter.
	**/
	public String toString() {
		if (nf != null && theParam != null)
			return nf.format(theParam.getValue());
		return super.toString();
	}


	/**
	*  Defines a listener for our unit menu combo box.
	**/
	protected class UnitMenuListener extends Object implements ActionListener {

		public void actionPerformed( ActionEvent evt ) {
			
			JComboBox unitList = (JComboBox)evt.getSource();
			Unit selectedUnit = (Unit)unitList.getSelectedItem();
			
			if ( !selectedUnit.equals(theParam.getUnits()) ) {
			
				//	Change units to whatever the user selected.
				try {
				
					theParam.changeUnits(selectedUnit);
					ParamInput.this.resetText();
					
				} catch( CannotConvertException e ) {
					/* Can't happen if availUnits are good! */
				}
			}
			
		}

	}
	
	
	/**
	*  Defines a listener for our text field that is called whenever the user
	*  types in a new number and presses enter.
	**/
	protected class TextFieldListener extends Object implements ActionListener {

		public  void actionPerformed( ActionEvent e ) {
			ParamInput.this.getInput();
		}
	}
	
	
	/**
	*  Defines a listener for our text field that is called whenever the
	*  focus changes off a particular text field.
	**/
	protected class TextFieldFocusL extends Object implements FocusListener {

		public  void focusGained( FocusEvent e ) { }

		public  void focusLost( FocusEvent e ) {
			ParamInput.this.getInput();
		}

	}

}


