/*
*   ParamMethod - Object that contains a method used to calculate an AutoParam.
*
*   Copyright (C) 1998-2004 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Lesser General Public License for more details.
*
*   You should have received a copy of the GNU Lesser General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*   Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.tools.ParamTools;



/**
*  An abstract class that serves as the base class for various
*  parameter method objects.  Parameter method objects contain a method
*  for calculating a given parameter's float value with a given list
*  of input parameter values.
*
*  <p>  Modified by:  Joseph A. Huwaldt  </p>
*
*  @author   Joseph A. Huwaldt   Date: February 1, 1998
*  @version  November 6, 2004
**/
public abstract class ParamMethod implements java.io.Serializable {

	/**
	*  Array to hold labels of input parameters.
	**/
	protected String[] inputLabels;

	/**
	*  The user of this class should supply a routine that calculates
	*  the parameter value with a given set of input values.  If
	*  an exception occures that can't be handled in calc(), it may be
	*  thrown.  It will be intercepted and reported back to the user
	*  by the AutoParamInput class.
	**/
	public abstract float calc( float[] inputs ) throws Exception;

	/**
	*  Returns true if the inputs for this parameter are appropriate
	*  for this method.  Normally, "valid" means that the values for the input
	*  parameters are within a certain range.
	*  The default implementation always returns true.
	**/
	public boolean validInputs(ParamDatabase pdb, String[] inputs) {
		return true;
	}
	
	/**
	*  Returns the number of inputs required by this method (the
	*  number of input parameter labels passed to the constructor when
	*  this object was instantiated).
	*
	*  @return  Returns the number of input values required by this method.
	**/
	public int getNumInputs() {
		if ( inputLabels != null )
			return inputLabels.length;
		
		return 0;
	}

	/**
	*  Returns the label of the input parameter specified by "index".
	*
	*  @param   index  Index into the input label array for the label to be returned.
	*  @return  Returns the label for the input parameter specified.
	*  @exception  ArrayIndexOutOfBoundsException  Thrown if index is < 0 or greater
	*              than the number of parameters available.
	**/
	public String getInputLabel( int index ) throws ArrayIndexOutOfBoundsException {
		return inputLabels[index];
	}

	/**
	*  Returns the array of all the input parameter labels.
	*
	*  @return  The array of all input parameter label strings is returned.
	**/
	public String[] getAllLabels() {
		return inputLabels;
	}

}


