/*
*   AngularUnit - Base class for all plane angle unit objects.
*
*   Copyright (C) 1997-2004 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*   You should have received a copy of the GNU Lesser General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*   Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.tools.units;


/**
*  An abstract class that serves as the base class for various
*  plane angle units such as degrees, and radians.
*  The reference unit for this class is "Radians".
*
*  <p>  Modified by:  Joseph A. Huwaldt  </p>
*
*  @author   Joseph A. Huwaldt   Date: December 18, 1997
*  @version  November 6, 2004
**/
public abstract class AngularUnit extends Unit {

	/**
	*  Converts a value with the current units to a value with
	*  the new units specified.
	*
	*  @param   value    The value in the "currentUnits" to be converted to "newUnit".
	*  @param   newUnit  The new unit system to convert value to.
	*  @return  The value converted to the new units.
	**/
	public double convert( double value, Unit newUnit ) throws CannotConvertException {
		if ( ! (newUnit instanceof AngularUnit) ) {
			String oldName = this.getClass().getName();
			String newName = newUnit.getClass().getName();
			throw new CannotConvertException( oldName + " to " + newName );
		}
		value *= newUnit.factor() / this.factor();
		return value;
	}

	/**
	*  Returns the reference unit for this unit type.
	*
	*  @return  The reference unit for this type of unit object.
	**/
	public Unit getRefUnit() {
		return Radians .getInstance();
	}

}


