/*
*   BrowserHelp - Provide "help" information to a program by opening an
*                 HTML document using a web browser.
*
*   Copyright (C) 1999-2004 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*   You should have received a copy of the GNU Lesser General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*   Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.util;


import java.applet.Applet;
import java.net.URL;
import java.io.IOException;

import com.Ostermiller.util.Browser;


/**
*  A class that provides "help" information to an application
*  or applet by opening a selected HTML page in a web browser.
*
*  <p>  Modified by:  Joseph A. Huwaldt    </p>
*
*  @author  Joseph A. Huwaldt   Date: October 7, 1999
*  @version November 6, 2004
**/
public class BrowserHelp implements HelpSystem, java.io.Serializable {

    static {
		//  Initialize the class used to open web pages in the user's default browser.
		Browser.init();
	}
	
	/**
	*  URL to web page with "help/info" information.
	**/
	private URL helpURL = null;
	
	/**
	*  The applet that is using this help system
	*  (if any).  Used for communicating with the web
	*  browser that this applet is running inside of.
	**/
	private	transient Applet helpApplet=null;
	

	//-----------------------------------------------------------------------------
	/**
	*  Dont' allow users access to the default constructor.
	**/
	protected BrowserHelp() { }

	/**
	*  Constructor that accepts the URL pointing to help information
	*  to show.  This constructor is intended for use in an application
	*  where the applet methods are not available.
	*
	*  @param  helpURL    The URL pointing to help information to show in a
	*                     web browser.
	**/
	public BrowserHelp( URL helpURL ) {
		this.helpURL = helpURL;
	}

	/**
	*  Constructor that accepts the URL pointing to help information
	*  to show and a reference to the applet that this object is running
	*  inside of.  The applet reference is required in order to communicate
	*  with the web browser.
	*
	*  @param  helpURL    The URL pointing to help information to show in a
	*                     web browser.
	*  @param  theApplet  Reference to the applet that this object is running
	*                     inside of.
	**/
	public BrowserHelp(URL helpURL, Applet theApplet) {
		this.helpApplet = theApplet;
		this.helpURL = helpURL;
	}

	//-----------------------------------------------------------------------------
	/**
	*  Method that displays the stored URL in a web browser when called.
	**/
	public void showHelp() {
	
		// If help is available, show the specified URL in a web browser.
		if ( helpAvailable() ) {
			if (helpApplet != null)
				// If we are in an applet show the help on it's own page.
				helpApplet.getAppletContext().showDocument(helpURL, "_self");
			else
				// If in application, launch browser and show page.
				showDocument( helpURL );
		}
		
	}

	/**
	*  Returns true if help information is available, false if it is not.
	*
	*  @return Returns true if help information is available, false otherwise.
	**/
	public boolean helpAvailable() {
		return (helpURL != null);
	}

	/**
	*  Open the specified URL in the default web browser.
	**/
    private void showDocument( URL url ) {
    
		// Launch a web browser and display the requested URL.
		try  {
			Browser.displayURL(url.toString());
		} catch( IOException e ) {
			e.printStackTrace();
		}

	}
    

}


